@@ -1,18 +1,7 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001-2002 Matthew W. Lefkowitz
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# -*- test-case-name: twisted.conch.test.test_conch -*-
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 # 
 
 """This module contains the implementation of SSHSession, which (by default)
@@ -36,58 +25,69 @@ class SSHSession(channel.SSHChannel):
     def __init__(self, *args, **kw):
         channel.SSHChannel.__init__(self, *args, **kw)
         self.buf = ''
-        self.session = ISession(self.avatar)
+        self.client = None
+        self.session = None
 
     def request_subsystem(self, data):
-        subsystem = common.getNS(data)[0]
+        subsystem, ignored= common.getNS(data)
+        log.msg('asking for subsystem "%s"' % subsystem)
         client = self.avatar.lookupSubsystem(subsystem, data)
         if client:
             pp = SSHSessionProcessProtocol(self)
             proto = wrapProcessProtocol(pp)
             client.makeConnection(proto)
             pp.makeConnection(wrapProtocol(client))
-            log.msg('starting subsystem %s'%subsystem)
             self.client = pp
             return 1
         else:
-            log.msg('failed to get subsystem %s'%subsystem)
+            log.msg('failed to get subsystem')
             return 0
 
     def request_shell(self, data):
+        log.msg('getting shell')
+        if not self.session:
+            self.session = ISession(self.avatar)
         try:
-            self.client = SSHSessionProcessProtocol(self)
-            self.session.openShell(self.client)
+            pp = SSHSessionProcessProtocol(self)
+            self.session.openShell(pp)
         except:
-            log.msg('error getting shell:')
             log.deferr()
             return 0
         else:
+            self.client = pp
             return 1
 
     def request_exec(self, data):
+        if not self.session:
+            self.session = ISession(self.avatar)
         f,data = common.getNS(data)
+        log.msg('executing command "%s"' % f)
         try:
-            self.client = SSHSessionProcessProtocol(self)
-            self.session.execCommand(self.client, f)
+            pp = SSHSessionProcessProtocol(self)
+            self.session.execCommand(pp, f)
         except:
-            log.msg('error executing command: %s' % f)
             log.deferr()
             return 0
         else:
+            self.client = pp
             return 1
 
     def request_pty_req(self, data):
+        if not self.session:
+            self.session = ISession(self.avatar)
         term, windowSize, modes = parseRequest_pty_req(data)
+        log.msg('pty request: %s %s' % (term, windowSize))
         try:
             self.session.getPty(term, windowSize, modes) 
         except:
-            log.msg('error getting pty')
             log.err()
             return 0
         else:
             return 1
 
     def request_window_change(self, data):
+        if not self.session:
+            self.session = ISession(self.avatar)
         import fcntl, tty
         winSize = parseRequest_window_change(data)
         try:
@@ -100,7 +100,7 @@ class SSHSession(channel.SSHChannel):
             return 1
 
     def dataReceived(self, data):
-        if not hasattr(self, 'client'):
+        if not self.client:
             #self.conn.sendClose(self)
             self.buf += data
             return
@@ -108,19 +108,25 @@ class SSHSession(channel.SSHChannel):
 
     def extReceived(self, dataType, data):
         if dataType == connection.EXTENDED_DATA_STDERR:
-            if hasattr(self, 'client') and hasattr(self.client.transport, 'writeErr'):
+            if self.client and hasattr(self.client.transport, 'writeErr'):
                 self.client.transport.writeErr(data)
         else:
             log.msg('weird extended data: %s'%dataType)
 
-    def closed(self):
-        self.session.closed()
-
     def eofReceived(self):
-        self.loseConnection() # don't know what to do with this
+        if self.session:
+            self.session.eofReceived()
+
+    def closed(self):
+        if self.session:
+            self.session.closed()
+
+    #def closeReceived(self):
+    #    self.loseConnection() # don't know what to do with this
 
     def loseConnection(self):
-        self.client.transport.loseConnection()
+        if self.client:
+            self.client.transport.loseConnection()
         channel.SSHChannel.loseConnection(self)
 
 class _ProtocolWrapper(protocol.ProcessProtocol):
@@ -155,7 +161,7 @@ class _DummyTransport:
     
 def wrapProcessProtocol(inst):
     if isinstance(inst, protocol.Protocol):
-        return ProtocolWrapper(inst)
+        return _ProtocolWrapper(inst)
     else:
         return inst
 
@@ -216,9 +222,9 @@ def parseRequest_pty_req(data):
     """
     term, rest = common.getNS(data)
     cols, rows, xpixel, ypixel = struct.unpack('>4L', rest[: 16])
-    modes = common.getNS(rest[16:])[0]
+    modes, ignored= common.getNS(rest[16:])
     winSize = (rows, cols, xpixel, ypixel)
-    modes = [(ord(modes[i]), struct.unpack('>L', modes[i+1: i+5])[0])for i in range(0, len(modes)-1, 5)]
+    modes = [(ord(modes[i]), struct.unpack('>L', modes[i+1: i+5])[0]) for i in range(0, len(modes)-1, 5)]
     return term, winSize, modes
 
 def packRequest_pty_req(term, (rows, cols, xpixel, ypixel), modes):
