@@ -1,20 +1,8 @@
-# -*- test-case-name: twisted.test.test_conch -*-
+# -*- test-case-name: twisted.conch.test.test_conch -*-
 #
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001-2002 Matthew W. Lefkowitz
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 # 
 
 """The lowest level SSH protocol.  This handles the key negotiation, the encryption and the compression.
@@ -107,8 +95,7 @@ class SSHTransportBase(protocol.Protocol):
     def sendPacket(self, messageType, payload):
         payload = chr(messageType)+payload
         if self.outgoingCompression:
-            payload = self.outgoingCompression.compress(payload)
-            payload = payload+self.outgoingCompression.flush(2)
+            payload = self.outgoingCompression.compress(payload) + self.outgoingCompression.flush(2)
         if self.currentEncryptions:
             bs = self.currentEncryptions.enc_block_size
         else:
@@ -117,18 +104,13 @@ class SSHTransportBase(protocol.Protocol):
         lenPad = bs-(totalSize%bs)
         if lenPad < 4:
             lenPad = lenPad+bs
-        randomPad = entropy.get_bytes(lenPad)
-        packet = struct.pack('!LB', 1+len(payload)+lenPad, lenPad)+ \
-                payload+randomPad
+        packet = struct.pack('!LB', totalSize+lenPad-4, lenPad)+ \
+                payload+entropy.get_bytes(lenPad)
         assert len(packet)%bs == 0, '%s extra bytes in packet'%(len(packet)%bs)
         if self.currentEncryptions:
-            encPacket = self.currentEncryptions.encrypt(packet)
-            assert len(encPacket) == len(packet), '%s %s'%(len(encPacket), len(packet))
+            encPacket = self.currentEncryptions.encrypt(packet) + self.currentEncryptions.makeMAC(self.outgoingPacketSequence, packet)
         else:
             encPacket = packet
-        if self.currentEncryptions:
-            d = self.currentEncryptions.makeMAC(self.outgoingPacketSequence, packet)
-            encPacket = encPacket+d
         self.transport.write(encPacket)
         self.outgoingPacketSequence+=1
 
@@ -203,7 +185,8 @@ class SSHTransportBase(protocol.Protocol):
                     log.msg(repr(packet[1:]))
                     self.sendUnimplemented()
             elif self.service:
-                self.service.packetReceived(ord(packet[0]), packet[1:])
+                log.callWithLogger(self.service, self.service.packetReceived,
+                                                 ord(packet[0]), packet[1:])
             else:
                 log.msg("couldn't handle %s"%messageNum)
                 log.msg(repr(packet[1:]))
@@ -267,9 +250,9 @@ class SSHTransportBase(protocol.Protocol):
         if self.currentEncryptions == None:
             return 0
         elif direction == "out":
-            return self.currentEncryptions.outCip != None
+            return bool(self.currentEncryptions.enc_block_size)
         elif direction == "in":
-            return self.currentEncryptions.outCip != None
+            return bool(self.currentEncryptions.dec_block_size)
         elif direction == "both":
             return self.isEncrypted("in")and self.isEncrypted("out")
         else:
@@ -289,6 +272,9 @@ class SSHTransportBase(protocol.Protocol):
         else:
             raise TypeError, 'direction must be "out", "in", or "both"'
 
+    def loseConnection(self):
+        self.sendDisconnect(DISCONNECT_CONNECTION_LOST, "user closed connection")
+
 class SSHServerTransport(SSHTransportBase):
     isClient = 0
     def ssh_KEXINIT(self, packet):
@@ -587,7 +573,7 @@ class SSHClientTransport(SSHTransportBase):
     def ssh_NEWKEYS(self, packet):
         if packet != '':
             self.sendDisconnect(DISCONNECT_PROTOCOL_ERROR, "NEWKEYS takes no data")
-        if not hasattr(self.nextEncryptions, 'outCip'):
+        if not self.nextEncryptions.enc_block_size:
             self._gotNewKeys = 1
             return
         self.currentEncryptions = self.nextEncryptions
@@ -649,12 +635,10 @@ class SSHCiphers:
         'blowfish-ctr':('Blowfish', 16, 1),
         'idea-ctr':('IDEA', 16, 1),
         'cast128-ctr':('CAST', 16, 1),
-        'none':(None, None), 
      }
     macMap = {
         'hmac-sha1': 'sha', 
         'hmac-md5': 'md5', 
-        'none': None, 
      }
 
     def __init__(self, outCip, inCip, outMac, inMac):
@@ -662,15 +646,19 @@ class SSHCiphers:
         self.inCipType = inCip
         self.outMacType = outMac
         self.inMacType = inMac
+        self.enc_block_size = 0
+        self.dec_block_size = 0
 
     def setKeys(self, outIV, outKey, inIV, inKey, outInteg, inInteg):
-        self.outCip = self._getCipher(self.outCipType, outIV, outKey)
-        self.enc_block_size = self.outCip.block_size
-        self.inCip = self._getCipher(self.inCipType, inIV, inKey)
-        self.dec_block_size = self.inCip.block_size
+        o = self._getCipher(self.outCipType, outIV, outKey)
+        self.encrypt = o.encrypt
+        self.enc_block_size = o.block_size
+        o = self._getCipher(self.inCipType, inIV, inKey)
+        self.decrypt = o.decrypt
+        self.dec_block_size = o.block_size
         self.outMAC = self._getMAC(self.outMacType, outInteg)
         self.inMAC = self._getMAC(self.inMacType, inInteg)
-        self.verify_digest_size = self.inMAC[2]
+        self.verify_digest_size = self.inMAC[3]
 
     def _getCipher(self, cip, iv, key):
         modName, keySize, counterMode = self.cipherMap[cip]
@@ -690,26 +678,28 @@ class SSHCiphers:
         else:
             ds = mod.digest_size
         key = key[: ds]+'\x00'*(64-ds)
-        return mod, key, ds
+        i = XOR.new('\x36').encrypt(key)
+        o = XOR.new('\x5c').encrypt(key)
+        return mod, i,o, ds
 
     def encrypt(self, blocks):
-        return self.outCip and self.outCip.encrypt(blocks) or blocks
+        return blocks
 
     def decrypt(self, blocks):
-        return self.inCip and self.inCip.decrypt(blocks) or blocks
+        return blocks
 
     def makeMAC(self, seqid, data):
         data = struct.pack('>L', seqid)+data
-        mod, key, ds = self.outMAC
-        inner = mod.new(XOR.new('\x36').encrypt(key)+data)
-        outer = mod.new(XOR.new('\x5c').encrypt(key)+inner.digest())
+        mod, i, o, ds = self.outMAC
+        inner = mod.new(i+data)
+        outer = mod.new(o+inner.digest())
         return outer.digest()
 
     def verify(self, seqid, data, mac):
         data = struct.pack('>L', seqid)+data
-        mod, key, ds = self.inMAC
-        inner = mod.new(XOR.new('\x36').encrypt(key)+data)
-        outer = mod.new(XOR.new('\x5c').encrypt(key)+inner.digest())
+        mod, i,o, ds = self.inMAC
+        inner = mod.new(i+data)
+        outer = mod.new(o+inner.digest())
         return mac == outer.digest()
 
 class _Counter:
@@ -778,8 +768,7 @@ DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE = 14
 DISCONNECT_ILLEGAL_USER_NAME = 15
 
 messages = {}
-import transport
-for v in dir(transport):
-    if v[: 4] == 'MSG_':
-        messages[getattr(transport, v)] = v # doesn't handle doubles
+for name, value in globals().items():
+    if name.startswith('MSG_'):
+        messages[value] = name
 
