@@ -1,18 +1,6 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001-2002 Matthew W. Lefkowitz
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 # 
 
 """Implementation of the ssh-userauth service.
@@ -104,14 +92,20 @@ class SSHUserAuthServer(service.SSHService):
         d.addErrback(self._ebBadAuth)
 
     def _cbFinishedAuth(self, (interface, avatar, logout)):
+        self.transport.isAuthorized = True
+        service = self.transport.factory.getService(self.transport, 
+                self.nextService)
+        if not service:
+            raise error.ConchError('could not get next service: %s' 
+                                  % self.nextService)
         log.msg('%s authenticated with %s' % (self.user, self.method))
-        self.transport.sendPacket(MSG_USERAUTH_SUCCESS, '')
-        self.transport.avatar = avatar
-        self.transport.logoutFunction = logout
         if self.cancelLoginTimeout:
             self.cancelLoginTimeout.cancel()
             self.cancelLoginTimeout = None
-        self.transport.setService(self.transport.factory.services[self.nextService]())
+        self.transport.sendPacket(MSG_USERAUTH_SUCCESS, '')
+        self.transport.avatar = avatar
+        self.transport.logoutFunction = logout
+        self.transport.setService(service())
 
     def _ebMaybeBadAuth(self, reason):
         reason.trap(error.NotEnoughAuthentication)
@@ -218,10 +212,11 @@ class SSHUserAuthClient(service.SSHService):
     def __init__(self, user, instance):
         self.user = user
         self.instance = instance
-        self.authenticatedWith = []
-        self.triedPublicKeys = []
 
     def serviceStarted(self):
+        self.authenticatedWith = []
+        self.triedPublicKeys = []
+        self.lastPublicKey = None
         self.askForAuth('none', '')
 
     def askForAuth(self, kind, extraData):
@@ -229,13 +224,18 @@ class SSHUserAuthClient(service.SSHService):
         self.transport.sendPacket(MSG_USERAUTH_REQUEST, NS(self.user) + \
                                   NS(self.instance.name) + NS(kind) + extraData)
     def tryAuth(self, kind):
+        kind = kind.replace('-', '_')
+        log.msg('trying to auth with %s' % kind)
         f= getattr(self,'auth_%s'%kind, None)
         if f:
             return f()
+
+    def _ebAuth(self, ignored, *args):
+        self.tryAuth('none')
         
     def ssh_USERAUTH_SUCCESS(self, packet):
         self.transport.setService(self.instance)
-        self.ssh_USERAUTH_SUCCESS = lambda *a: None # ignore these
+        #self.ssh_USERAUTH_SUCCESS = lambda *a: None # ignore these
 
     def ssh_USERAUTH_FAILURE(self, packet):
         canContinue, partial = getNS(packet)
@@ -274,16 +274,25 @@ class SSHUserAuthClient(service.SSHService):
                 # this will fail, we'll move on
                 return
             d.addCallback(self._cbSignedData)
-            d.addErrback(self._ebSignedData)
+            d.addErrback(self._ebAuth)
         elif self.lastAuth == 'password':
             prompt, language, rest = getNS(packet, 2)
             self._oldPass = self._newPass = None
-            self.getPassword('Old Password: ').addCallbacks(self._setOldPass, self._errPass)
-            self.getPassword(prompt).addCallbacks(self._setNewPass, self._errPass)
+            self.getPassword('Old Password: ').addCallbacks(self._setOldPass, self._ebAuth)
+            self.getPassword(prompt).addCallbacks(self._setNewPass, self._ebAuth)
         elif self.lastAuth == 'keyboard-interactive':
-            # can't handle this in the client, so just try something else
-            self.askForAuth('none', '')
-            #return self.ssh_USERAUTH_INFO_RESPONSE(packet)
+            name, instruction, lang, data = getNS(packet, 3)
+            numPrompts = struct.unpack('!L', data[:4])[0]
+            data = data[4:]
+            prompts = []
+            for i in range(numPrompts):
+                prompt, data = getNS(data)
+                echo = bool(ord(data[0]))
+                data = data[1:]
+                prompts.append((prompt, echo))
+            d = self.getGenericAnswers(name, instruction, prompts)
+            d.addCallback(self._cbGenericAnswers)
+            d.addErrback(self._ebAuth)
 
     def _cbSignedData(self, signedData):
         publicKey = self.lastPublicKey
@@ -291,8 +300,10 @@ class SSHUserAuthClient(service.SSHService):
         self.askForAuth('publickey', '\xff' + NS(keyType) + NS(publicKey) + \
                         NS(signedData))
 
-    def _ebSignedData(self, ignored):
-        self.askForAuth('none', '')
+
+
+
+
 
     def _setOldPass(self, op):
         if self._newPass:
@@ -310,8 +321,11 @@ class SSHUserAuthClient(service.SSHService):
         else:
             self._newPass = np
 
-    def _errPass(self, reason):
-        self.askForAuth('none', '')
+    def _cbGenericAnswers(self, responses):
+        data = struct.pack('!L', len(responses))
+        for r in responses:
+            data += NS(r.encode('UTF8'))
+        self.transport.sendPacket(MSG_USERAUTH_INFO_RESPONSE, data)
 
     def auth_publickey(self):
         publicKey = self.getPublicKey()
@@ -329,11 +343,16 @@ class SSHUserAuthClient(service.SSHService):
     def auth_password(self):
         d = self.getPassword()
         if d:
-            d.addCallbacks(self._cbPassword, self._errPass)
+            d.addCallbacks(self._cbPassword, self._ebAuth)
             return 1
         else: # returned None, don't do password auth
             return 0
 
+    def auth_keyboard_interactive(self):
+        log.msg('authing with keyboard-interactive')
+        self.askForAuth('keyboard-interactive', NS('') + NS(''))
+        return 1
+
     def _cbPassword(self, password):
         self.askForAuth('password', '\x00'+NS(password))
     
@@ -360,7 +379,8 @@ class SSHUserAuthClient(service.SSHService):
 
         @rtype: C{str}/C{None}
         """
-        raise NotImplementedError
+        return None
+        #raise NotImplementedError
 
 
     def getPrivateKey(self):
@@ -371,7 +391,7 @@ class SSHUserAuthClient(service.SSHService):
 
         @rtype: C{Deferred}
         """
-        raise NotImplementedError
+        return defer.fail(NotImplementedError())
 
     def getPassword(self, prompt = None):
         """
@@ -382,7 +402,21 @@ class SSHUserAuthClient(service.SSHService):
         @type prompt: C{str}/C{None}
         @rtype: C{Deferred}
         """
-        raise NotImplementedError
+        return defer.fail(NotImplementedError())
+
+    def getGenericAnswers(self, name, instruction, prompts):
+        """
+        Returns a C{Deferred} with the responses to the promopts.
+
+        name is the name of the authentication currently
+        in progress.
+        instruction describes what the authentication wants.
+        prompts is a list of (prompt, echo) pairs.
+            prompt is the prompt to display.
+            echo is a C{bool}.  If False, do not display what the
+            user types for this prompt.
+        """
+        return defer.fail(NotImplementedError())
 
 MSG_USERAUTH_REQUEST          = 50
 MSG_USERAUTH_FAILURE          = 51
