@@ -1,24 +1,13 @@
 # -*- test-case-name: twisted.test.test_newcred -*-
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2003 Matthew W. Lefkowitza
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 from twisted.internet import defer
 from twisted.python import components, failure, log
 from twisted.cred import error, credentials
+from zope import interface
 
 class ICredentialsChecker(components.Interface):
     """I check sub-interfaces of ICredentials.
@@ -52,15 +41,26 @@ ANONYMOUS = ()
 
 
 class AllowAnonymousAccess:
-    __implements__ = ICredentialsChecker
+    interface.implements(ICredentialsChecker)
     credentialInterfaces = credentials.IAnonymous,
 
     def requestAvatarId(self, credentials):
         return defer.succeed(ANONYMOUS)
 
+components.backwardsCompatImplements(AllowAnonymousAccess)
 
 class InMemoryUsernamePasswordDatabaseDontUse:
-    __implements__ = ICredentialsChecker
+    """An extremely simple credentials checker.
+    
+    This is only of use in one-off test programs or examples which don't
+    want to focus too much on how credentials are verified.
+    
+    You really don't want to use this for anything else.  It is, at best, a
+    toy.  If you need a simple credentials checker for a real application,
+    see L{FilePasswordDB}.
+    """
+
+    interface.implements(ICredentialsChecker)
 
     credentialInterfaces = (credentials.IUsernamePassword,
         credentials.IUsernameHashedPassword)
@@ -86,6 +86,7 @@ class InMemoryUsernamePasswordDatabaseDontUse:
         else:
             return failure.Failure(error.UnauthorizedLogin())
 
+components.backwardsCompatImplements(InMemoryUsernamePasswordDatabaseDontUse)
 
 class FilePasswordDB:
     """A file-based, text-based username/password database.
@@ -100,7 +101,7 @@ class FilePasswordDB:
     IUsernameHashedPassword credentials will be checkable as well.
     """
 
-    __implements__ = (ICredentialsChecker,)
+    interface.implements(ICredentialsChecker)
 
     def __init__(self, filename, delim=':', usernameField=0, passwordField=1,
                  caseSensitive=True, hash=None):
@@ -201,6 +202,8 @@ class FilePasswordDB:
                 return defer.maybeDeferred(c.checkPassword, p
                     ).addCallback(self._cbPasswordMatch, u)
 
+components.backwardsCompatImplements(FilePasswordDB)
+
 # For backwards compatibility
 # Allow access as the old name.
 OnDiskUsernamePasswordDatabase = FilePasswordDB
