@@ -1,18 +1,7 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# -*- test-case-name: twisted.test.test_app -*-
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """This module is DEPRECATED.
 
@@ -27,6 +16,7 @@ import string
 import socket
 import types
 import warnings
+from zope.interface import implements
 
 try:
     import cPickle as pickle
@@ -40,7 +30,7 @@ except ImportError:
 
 # Twisted Imports
 from twisted.internet import interfaces
-from twisted.python import log
+from twisted.python import log, components
 from twisted.persisted import styles
 from twisted.python.runtime import platform
 from twisted.cred.authorizer import DefaultAuthorizer
@@ -50,7 +40,7 @@ from twisted.python.util import OrderedDict
 from twisted.python import context
 
 # Sibling Imports
-import main, defer, error
+import defer, error
 
 def encrypt(passphrase, data):
     import md5
@@ -70,7 +60,7 @@ def quieterWarning():
     quieterWarning = lambda: None
 
 class _AbstractServiceCollection:
-    __implements__ = (interfaces.IServiceCollection, )
+    implements(interfaces.IServiceCollection)
 
     def __init__(self):
         """Create an abstract service collection.
@@ -96,6 +86,9 @@ class _AbstractServiceCollection:
         """Remove a service from this collection."""
         del self.services[service.serviceName]
 
+components.backwardsCompatImplements(_AbstractServiceCollection)
+
+
 class ApplicationService(Accessor, styles.Versioned):
     """I am a service you can add to an application.
 
@@ -489,7 +482,6 @@ class Application(log.Logger, styles.Versioned,
     def upgradeToVersion2(self):
         """Version 2 Persistence Upgrade
         """
-        #self.resolver = main.DummyResolver()
         self.resolver = None # will be deleted in upgrade to v6
     
     def upgradeToVersion1(self):
@@ -869,7 +861,6 @@ class Application(log.Logger, styles.Versioned,
         reactor.addSystemEventTrigger('after', 'shutdown', self._afterShutDown)
         global theApplication
         theApplication = self
-        main.running = 1 # just in case
         log.callWithLogger(self, reactor.run, installSignalHandlers=installSignalHandlers)
 
 
