@@ -1,19 +1,7 @@
 # -*- test-case-name: twisted.test.test_internet -*-
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """Very basic functionality for a Reactor implementation.
 
@@ -23,10 +11,12 @@ Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 """
 
 import socket # needed only for sync-dns
+from zope.interface import implements
 
 import sys
 import warnings
-from bisect import insort
+import operator
+from heapq import heappush, heappop, heapreplace, heapify
 
 try:
     import fcntl
@@ -34,28 +24,30 @@ except ImportError:
     fcntl = None
 import traceback
 
-from twisted.internet.interfaces import IReactorCore, IReactorTime, IReactorUNIX, IReactorUNIXDatagram, IReactorThreads
-from twisted.internet.interfaces import IReactorTCP, IReactorUDP, IReactorSSL
-from twisted.internet.interfaces import IReactorProcess, IReactorPluggableResolver
+from twisted.internet.interfaces import IReactorCore, IReactorTime, IReactorThreads
+from twisted.internet.interfaces import IResolverSimple, IReactorPluggableResolver
 from twisted.internet.interfaces import IConnector, IDelayedCall
-from twisted.internet import main, error, abstract, defer
-from twisted.python import threadable, log, failure, reflect
+from twisted.internet import main, error, abstract, defer, threads
+from twisted.python import threadable, log, failure, reflect, components
 from twisted.python.runtime import seconds
 from twisted.internet.defer import Deferred, DeferredList
 from twisted.persisted import styles
 
+
 class DelayedCall(styles.Ephemeral):
 
-    __implements__ = IDelayedCall
+    implements(IDelayedCall)
     # enable .debug to record creator call stack, and it will be logged if
     # an exception occurs while the function is being run
     debug = False
+    _str = None
 
     def __init__(self, time, func, args, kw, cancel, reset):
         self.time, self.func, self.args, self.kw = time, func, args, kw
         self.resetter = reset
         self.canceller = cancel
         self.cancelled = self.called = 0
+        self.delayed_time = 0
         if self.debug:
             self.creator = traceback.format_stack()[:-2]
 
@@ -66,7 +58,7 @@ class DelayedCall(styles.Ephemeral):
         @return: The number of seconds after the epoch at which this call is
         scheduled to be made.
         """
-        return self.time
+        return self.time + self.delayed_time
 
     def cancel(self):
         """Unschedule this call
@@ -83,6 +75,9 @@ class DelayedCall(styles.Ephemeral):
         else:
             self.canceller(self)
             self.cancelled = 1
+            if self.debug:
+                self._str = str(self)
+            del self.func, self.args, self.kw
 
     def reset(self, secondsFromNow):
         """Reschedule this call for a different time
@@ -99,8 +94,13 @@ class DelayedCall(styles.Ephemeral):
         elif self.called:
             raise error.AlreadyCalled
         else:
-            self.time = seconds() + secondsFromNow
-            self.resetter(self)
+            new_time = seconds() + secondsFromNow
+            if new_time < self.time:
+                self.delayed_time = 0
+                self.time = new_time
+                self.resetter(self)
+            else:
+                self.delayed_time = new_time - self.time
 
     def delay(self, secondsLater):
         """Reschedule this call for a later time
@@ -117,8 +117,14 @@ class DelayedCall(styles.Ephemeral):
         elif self.called:
             raise error.AlreadyCalled
         else:
-            self.time += secondsLater
-            self.resetter(self)
+            self.delayed_time += secondsLater
+            if self.delayed_time < 0:
+                self.activate_delay()
+                self.resetter(self)
+
+    def activate_delay(self):
+        self.time += self.delayed_time
+        self.delayed_time = 0
 
     def active(self):
         """Determine whether this call is still pending
@@ -129,42 +135,111 @@ class DelayedCall(styles.Ephemeral):
         """
         return not (self.cancelled or self.called)
 
-    def __lt__(self, other):
-        # Order reversed for efficiency concerns, see below
-        return self.time >= other.time
+    def __le__(self, other):
+        return self.time <= other.time
 
     def __str__(self):
-        try:
+        if self._str is not None:
+            return self._str
+        if hasattr(self.func, 'func_name'):
             func = self.func.func_name
-            try:
+            if hasattr(self.func, 'im_class'):
                 func = self.func.im_class.__name__ + '.' + func
-            except:
-                func = self.func
-                if hasattr(func, 'func_code'):
-                    func = func.func_code # func_code's repr sometimes has more useful info
-        except:
+        else:
             func = reflect.safe_repr(self.func)
-        return "<DelayedCall %s [%ss] called=%s cancelled=%s %s%s>" % (
-            id(self), self.time - seconds(), self.called, self.cancelled, func,
-            reflect.safe_repr(self.args))
 
+        L = ["<DelayedCall %s [%ss] called=%s cancelled=%s %s%s" % (
+                id(self), self.time - seconds(), self.called, self.cancelled, func,
+                reflect.safe_repr(self.args))]
+
+        if self.debug:
+            L.append("\n\ntraceback at creation: \n\n%s" % ('    '.join(self.creator)))
+        L.append('>')
+
+        return "".join(L)
+
+
+components.backwardsCompatImplements(DelayedCall)
+
+class ThreadedResolver:
+    implements(IResolverSimple)
+
+    def __init__(self, reactor):
+        self.reactor = reactor
+        self._runningQueries = {}
+
+    def _fail(self, name, err):
+        err = error.DNSLookupError("address %r not found: %s" % (name, err))
+        return failure.Failure(err)
+
+    def _cleanup(self, name, lookupDeferred):
+        userDeferred, cancelCall = self._runningQueries[lookupDeferred]
+        del self._runningQueries[lookupDeferred]
+        userDeferred.errback(self._fail(name, "timeout error"))
+
+    def _checkTimeout(self, result, name, lookupDeferred):
+        try:
+            userDeferred, cancelCall = self._runningQueries[lookupDeferred]
+        except KeyError:
+            pass
+        else:
+            del self._runningQueries[lookupDeferred]
+            cancelCall.cancel()
+
+            if isinstance(result, failure.Failure):
+                userDeferred.errback(self._fail(name, result.getErrorMessage()))
+            else:
+                userDeferred.callback(result)
+
+    def getHostByName(self, name, timeout = (1, 3, 11, 45)):
+        if timeout:
+            timeoutDelay = reduce(operator.add, timeout)
+        else:
+            timeoutDelay = 60
+        userDeferred = defer.Deferred()
+        lookupDeferred = threads.deferToThread(socket.gethostbyname, name)
+        cancelCall = self.reactor.callLater(
+            timeoutDelay, self._cleanup, name, lookupDeferred)
+        self._runningQueries[lookupDeferred] = (userDeferred, cancelCall)
+        lookupDeferred.addBoth(self._checkTimeout, name, lookupDeferred)
+        return userDeferred
+
+components.backwardsCompatImplements(ThreadedResolver)
+
+class BlockingResolver:
+    implements(IResolverSimple)
+
+    def getHostByName(self, name, timeout = (1, 3, 11, 45)):
+        try:
+            address = socket.gethostbyname(name)
+        except socket.error:
+            msg = "address %r not found" % (name,)
+            err = error.DNSLookupError(msg)
+            return defer.fail(err)
+        else:      
+            return defer.succeed(address)
+
+components.backwardsCompatImplements(BlockingResolver)
 
 class ReactorBase:
     """Default base class for Reactors.
     """
 
-    __implements__ = IReactorCore, IReactorTime, IReactorThreads, IReactorPluggableResolver
+    implements(IReactorCore, IReactorTime, IReactorThreads, IReactorPluggableResolver)
     installed = 0
 
     __name__ = "twisted.internet.reactor"
 
     def __init__(self):
+        self.threadCallQueue = []
         self._eventTriggers = {}
         self._pendingTimedCalls = []
+        self._newTimedCalls = []
+        self._cancellations = 0
         self.running = 0
         self.waker = None
-        self.resolver = None
         self.usingThreads = 0
+        self.resolver = ThreadedResolver(self)
         self.addSystemEventTrigger('during', 'shutdown', self.crash)
         self.addSystemEventTrigger('during', 'shutdown', self.disconnectAll)
         threadable.whenThreaded(self.initThreads)
@@ -177,28 +252,25 @@ class ReactorBase:
         import thread
         self.usingThreads = 1
         self.installWaker()
-        self.threadCallQueue = []
-
-    threadCallQueue = None
 
     def installWaker(self):
         raise NotImplementedError()
 
     def installResolver(self, resolver):
+        assert IResolverSimple.providedBy(resolver)
+        oldResolver = self.resolver
         self.resolver = resolver
+        return oldResolver
 
     def callFromThread(self, f, *args, **kw):
         """See twisted.internet.interfaces.IReactorThreads.callFromThread.
         """
         assert callable(f), "%s is not callable" % f
-        if threadable.isInIOThread():
-            self.callLater(0, f, *args, **kw)
-        else:
-            # lists are thread-safe in CPython, but not in Jython
-            # this is probably a bug in Jython, but until fixed this code
-            # won't work in Jython.
-            self.threadCallQueue.append((f, args, kw))
-            self.wakeUp()
+        # lists are thread-safe in CPython, but not in Jython
+        # this is probably a bug in Jython, but until fixed this code
+        # won't work in Jython.
+        self.threadCallQueue.append((f, args, kw))
+        self.wakeUp()
 
     def wakeUp(self):
         """Wake up the event loop."""
@@ -227,7 +299,7 @@ class ReactorBase:
     def removeAll(self):
         raise NotImplementedError
 
-    def resolve(self, name, timeout = 10):
+    def resolve(self, name, timeout = (1, 3, 11, 45)):
         """Return a Deferred that will resolve a hostname.
         """
         if not name:
@@ -235,18 +307,8 @@ class ReactorBase:
             return defer.succeed('0.0.0.0')
         if abstract.isIPAddress(name):
             return defer.succeed(name)
-        if self.resolver is None:
-            return self._internalResolve(name, timeout)
         return self.resolver.getHostByName(name, timeout)
 
-    def _internalResolve(self, name, timeout):
-        try:
-            address = socket.gethostbyname(name)
-        except socket.error:
-            return defer.fail(failure.Failure(error.DNSLookupError("address %r not found" % name)))
-        else:
-            return defer.succeed(address)
-
     # Installation.
 
     # IReactorCore
@@ -254,6 +316,8 @@ class ReactorBase:
     def stop(self):
         """See twisted.internet.interfaces.IReactorCore.stop.
         """
+        if not self.running:
+            raise RuntimeError, "can't stop reactor that isn't running"
         self.fireSystemEvent("shutdown")
 
     def crash(self):
@@ -265,19 +329,19 @@ class ReactorBase:
         """Handle a SIGINT interrupt.
         """
         log.msg("Received SIGINT, shutting down.")
-        self.callLater(0, self.stop)
+        self.callFromThread(self.stop)
 
     def sigBreak(self, *args):
         """Handle a SIGBREAK interrupt.
         """
         log.msg("Received SIGBREAK, shutting down.")
-        self.callLater(0, self.stop)
+        self.callFromThread(self.stop)
 
     def sigTerm(self, *args):
         """Handle a SIGTERM interrupt.
         """
         log.msg("Received SIGTERM, shutting down.")
-        self.callLater(0, self.stop)
+        self.callFromThread(self.stop)
 
     def disconnectAll(self):
         """Disconnect every reader, and writer in the system.
@@ -328,7 +392,10 @@ class ReactorBase:
                     callable(*args, **kw)
                 except:
                     log.deferr()
-
+        # now that we've called all callbacks, no need to store
+        # references to them anymore, in fact this can cause problems.
+        del self._eventTriggers[eventType]
+    
     def addSystemEventTrigger(self, _phase, _eventType, _f, *args, **kw):
         """See twisted.internet.interfaces.IReactorCore.addSystemEventTrigger.
         """
@@ -369,16 +436,33 @@ class ReactorBase:
         assert sys.maxint >= _seconds >= 0, \
                "%s is not greater than or equal to 0 seconds" % (_seconds,)
         tple = DelayedCall(seconds() + _seconds, _f, args, kw,
-                           self._pendingTimedCalls.remove,
-                           self._resetCallLater)
-        insort(self._pendingTimedCalls, tple)
+                           self._cancelCallLater,
+                           self._moveCallLaterSooner)
+        self._newTimedCalls.append(tple)
         return tple
 
-    def _resetCallLater(self, tple):
-        assert tple in self._pendingTimedCalls
-        self._pendingTimedCalls.remove(tple)
-        insort(self._pendingTimedCalls, tple)
-        return tple
+    def _moveCallLaterSooner(self, tple):
+        # Linear time find: slow.
+        heap = self._pendingTimedCalls
+        try:
+            pos = heap.index(tple)
+
+            # Move elt up the heap until it rests at the right place.
+            elt = heap[pos]
+            while pos != 0:
+                parent = (pos-1) // 2
+                if heap[parent] <= elt:
+                    break
+                # move parent down
+                heap[pos] = heap[parent]
+                pos = parent
+            heap[pos] = elt
+        except ValueError:
+            # element was not found in heap - oh well...
+            pass
+        
+    def _cancelCallLater(self, tple):
+        self._cancellations+=1
 
     def cancelCallLater(self, callID):
         """See twisted.internet.interfaces.IReactorTime.cancelCallLater.
@@ -389,17 +473,30 @@ class ReactorBase:
         callID.cancel()
 
     def getDelayedCalls(self):
-        return tuple(self._pendingTimedCalls)
+        """Return all the outstanding delayed calls in the system.
+        They are returned in no particular order.
+        This method is not efficient -- it is really only meant for
+        test cases."""
+        return [x for x in (self._pendingTimedCalls + self._newTimedCalls) if not x.cancelled]
 
+    def _insertNewDelayedCalls(self):
+        for call in self._newTimedCalls:
+            if call.cancelled:
+                self._cancellations-=1
+            else:
+                call.activate_delay()
+                heappush(self._pendingTimedCalls, call)
+        self._newTimedCalls = []
+        
     def timeout(self):
-        if self._pendingTimedCalls:
-            t = self._pendingTimedCalls[-1].time - seconds()
-            if t < 0:
-                t = 0
-            return t
-        else:
+        # insert new delayed calls to make sure to include them in timeout value
+        self._insertNewDelayedCalls()
+        
+        if not self._pendingTimedCalls:
             return None
-
+        
+        return max(0, self._pendingTimedCalls[0].time - seconds())
+        
     def runUntilCurrent(self):
         """Run all pending timed calls.
         """
@@ -415,9 +512,22 @@ class ReactorBase:
                     log.err()
                 count += 1
             del self.threadCallQueue[:count]
+        
+        # insert new delayed calls now
+        self._insertNewDelayedCalls()
+        
         now = seconds()
-        while self._pendingTimedCalls and (self._pendingTimedCalls[-1].time <= now):
-            call = self._pendingTimedCalls.pop()
+        while self._pendingTimedCalls and (self._pendingTimedCalls[0].time <= now):
+            call = heappop(self._pendingTimedCalls)
+            if call.cancelled:
+                self._cancellations-=1
+                continue
+            
+            if call.delayed_time > 0:
+                call.activate_delay()
+                heappush(self._pendingTimedCalls, call)
+                continue
+
             try:
                 call.called = 1
                 call.func(*call.args, **call.kw)
@@ -432,7 +542,13 @@ class ReactorBase:
                     e += "\n"
                     log.msg(e)
 
-
+        
+        if (self._cancellations > 50 and
+             self._cancellations > len(self._pendingTimedCalls) >> 1):
+            self._cancellations = 0
+            self._pendingTimedCalls = [x for x in self._pendingTimedCalls
+                                       if not x.cancelled]
+            heapify(self._pendingTimedCalls)
 
     # IReactorThreads
 
@@ -461,63 +577,7 @@ class ReactorBase:
             self._initThreadPool()
         self.threadpool.adjustPoolsize(maxthreads=size)
 
-    # backwards compatibility
-
-    def clientUNIX(self, address, protocol, timeout=30):
-        """Deprecated - use connectUNIX instead.
-        """
-        warnings.warn("clientUNIX is deprecated - use connectUNIX instead.",
-                      category=DeprecationWarning, stacklevel=2)
-        f = BCFactory(protocol)
-        self.connectUNIX(address, f, timeout)
-
-
-    def clientTCP(self, host, port, protocol, timeout=30):
-        """Deprecated - use connectTCP instead.
-        """
-        warnings.warn("clientTCP is deprecated - use connectTCP instead.",
-                      category=DeprecationWarning, stacklevel=2)
-        f = BCFactory(protocol)
-        self.connectTCP(host, port, f, timeout)
-        return f
-
-    def clientSSL(self, host, port, protocol, contextFactory, timeout=30):
-        """Deprecated - use connectSSL instead.
-        """
-        warnings.warn("clientSSL is deprecated - use connectSSL instead.",
-                      category=DeprecationWarning, stacklevel=2)
-        f = BCFactory(protocol)
-        self.connectSSL(host, port, f, contextFactory, timeout)
-
-
-
-
-from protocol import ClientFactory
-
-class BCFactory(ClientFactory):
-    """Factory for backwards compatability with old clientXXX APIs."""
-
-    def __init__(self, protocol):
-        self.protocol = protocol
-        self.connector = None
-
-    def startedConnecting(self, connector):
-        self.connector = connector
-
-    def loseConnection(self):
-        if self.connector:
-            self.connector.stopConnecting()
-        elif self.protocol:
-            self.protocol.transport.loseConnection()
-
-    def buildProtocol(self, addr):
-        self.connector = None
-        return self.protocol
-
-    def clientConnectionFailed(self, connector, reason):
-        self.connector = None
-        self.protocol.connectionFailed()
-        self.protocol = None
+components.backwardsCompatImplements(ReactorBase)
 
 
 class BaseConnector(styles.Ephemeral):
@@ -526,7 +586,7 @@ class BaseConnector(styles.Ephemeral):
     State can be: "connecting", "connected", "disconnected"
     """
 
-    __implements__ = IConnector,
+    implements(IConnector)
 
     timeoutID = None
     factoryStarted = 0
@@ -568,7 +628,7 @@ class BaseConnector(styles.Ephemeral):
         del self.transport
 
     def cancelTimeout(self):
-        if self.timeoutID:
+        if self.timeoutID is not None:
             try:
                 self.timeoutID.cancel()
             except ValueError:
@@ -582,6 +642,7 @@ class BaseConnector(styles.Ephemeral):
 
     def connectionFailed(self, reason):
         self.cancelTimeout()
+        self.transport = None
         self.state = "disconnected"
         self.factory.clientConnectionFailed(self, reason)
         if self.state == "disconnected":
@@ -597,6 +658,11 @@ class BaseConnector(styles.Ephemeral):
             self.factory.doStop()
             self.factoryStarted = 0
 
+    def getDestination(self):
+        raise NotImplementedError, "implement in subclasses"
+
+components.backwardsCompatImplements(BaseConnector)
+
 
 class BasePort(abstract.FileDescriptor):
     """Basic implementation of a ListeningPort."""
@@ -618,4 +684,4 @@ class BasePort(abstract.FileDescriptor):
         raise RuntimeError, "doWrite called on a %s" % reflect.qual(self.__class__)
 
 
-__all__ = ["ReactorBase"]
+__all__ = []
