@@ -1,20 +1,8 @@
 # -*- test-case-name: twisted.test.test_defer -*-
 #
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """Support for results that aren't immediately available.
 
@@ -23,12 +11,13 @@ API Stability: stable
 Maintainer: U{Glyph Lefkowitz<mailto:glyph@twistedmatrix.com>}
 """
 
-from __future__ import nested_scopes
+from __future__ import nested_scopes, generators
 import traceback
+import warnings
 
 # Twisted imports
 from twisted.python import log, failure
-
+from twisted.python.util import unsignedID
 
 class AlreadyCalledError(Exception):
     pass
@@ -119,7 +108,6 @@ def maybeDeferred(f, *args, **kw):
     """
     deferred = None
     if isinstance(f, Deferred) or f is None:
-        import warnings
         warnings.warn("First argument to maybeDeferred() should no longer be a Deferred or None.  Just pass the function and the arguments.", DeprecationWarning, stacklevel=2)
         deferred = f or Deferred()
         f = args[0]
@@ -156,6 +144,14 @@ def timeout(deferred):
 def passthru(arg):
     return arg
 
+def setDebugging(on):
+    """Enable or disable Deferred debugging.
+
+    When debugging is on, the call stacks from creation and invocation are
+    recorded, and added to any AlreadyCalledErrors we raise.
+    """
+    Deferred.debug=bool(on)
+
 class Deferred:
     """This is a callback which will be put off until later.
 
@@ -171,19 +167,21 @@ class Deferred:
     For more information about Deferreds, see doc/howto/defer.html or
     U{http://www.twistedmatrix.com/documents/howto/defer}
     """
-
     called = 0
     default = 0
     paused = 0
     timeoutCall = None
-    # enable .debug to record creation/first-invoker call stacks, and they
-    # will be added to any AlreadyCalledErrors we raise
+    _debugInfo = None
+
+    # Keep this class attribute for now, for compatibility with code that
+    # sets it directly.
     debug = False
 
     def __init__(self):
         self.callbacks = []
         if self.debug:
-            self.creator = traceback.format_stack()[:-1]
+            self._debugInfo = DebugInfo()
+            self._debugInfo.creator = traceback.format_stack()[:-1]
 
     def addCallbacks(self, callback, errback=None,
                      callbackArgs=None, callbackKeywords=None,
@@ -200,6 +198,10 @@ class Deferred:
             self.callbacks[-1] = cbs
         else:
             self.callbacks.append(cbs)
+        if asDefaults:
+            # what the heck is this crappy argument for?
+            warnings.warn("The 'asDefaults' argument will be going away soon. Has anyone ever actually used it?", DeprecationWarning, stacklevel=2)
+
         self.default = asDefaults
         if self.called:
             self._runCallbacks()
@@ -295,12 +297,12 @@ class Deferred:
 
     def _startRunCallbacks(self, result):
         if self.called:
-            if not self.debug:
-                raise AlreadyCalledError
-            extra = "\n" + self._debugInfo()
-            raise AlreadyCalledError(extra)
+            if self.debug:
+                extra = "\n" + self._debugInfo._getDebugTracebacks()
+                raise AlreadyCalledError(extra)
+            raise AlreadyCalledError
         if self.debug:
-            self.invoker = traceback.format_stack()[:-2]
+            self._debugInfo.invoker = traceback.format_stack()[:-2]
         self.called = True
         self.result = result
         if self.timeoutCall:
@@ -308,22 +310,10 @@ class Deferred:
                 self.timeoutCall.cancel()
             except:
                 pass
-            # Avoid reference cycles, because this object defines __del__
+
             del self.timeoutCall
         self._runCallbacks()
 
-    def _debugInfo(self):
-        info = ''
-        if hasattr(self, "creator"):
-            info += " C: Deferred was created:\n C:"
-            info += "".join(self.creator).rstrip().replace("\n","\n C:")
-            info += "\n"
-        if hasattr(self, "invoker"):
-            info += " I: First Invoker was:\n I:"
-            info += "".join(self.invoker).rstrip().replace("\n","\n I:")
-            info += "\n"
-        return info
-
     def _runCallbacks(self):
         if not self.paused:
             cb = self.callbacks
@@ -350,14 +340,20 @@ class Deferred:
                         break
                 except:
                     self.result = failure.Failure()
+
         if isinstance(self.result, failure.Failure):
             self.result.cleanFailure()
-
+            if self._debugInfo is None:
+                self._debugInfo = DebugInfo()
+            self._debugInfo.failResult = self.result
+        else:
+            if self._debugInfo is not None:
+                self._debugInfo.failResult = None
 
     def arm(self):
         """This method is deprecated.
         """
-        pass
+        warnings.warn("Deferred.arm is deprecated, and does nothing. You should stop calling it.", DeprecationWarning, stacklevel=2)
 
     def setTimeout(self, seconds, timeoutFunc=timeout, *args, **kw):
         """Set a timeout function to be triggered if I am not called.
@@ -368,12 +364,11 @@ class Deferred:
         @param timeoutFunc: will receive the Deferred and *args, **kw as its
         arguments.  The default timeoutFunc will call the errback with a
         L{TimeoutError}.
-
-        DON'T USE THIS! It's a bad idea! Use a function called by reactor.callLater instead
-        to accomplish the same thing!
-
-        YOU HAVE BEEN WARNED!
         """
+        warnings.warn(
+            "Deferred.setTimeout is deprecated.  Look for timeout "
+            "support specific to the API you are using instead.",
+            DeprecationWarning, stacklevel=2)
 
         if self.called:
             return
@@ -385,19 +380,41 @@ class Deferred:
             lambda: self.called or timeoutFunc(self, *args, **kw))
         return self.timeoutCall
 
-    armAndErrback = errback
-    armAndCallback = callback
-    armAndChain = chainDeferred
+    def armAndErrback(self, fail=None):
+        warnings.warn("Deferred.armAndErrback is deprecated. You should be calling .errback instead.", DeprecationWarning, stacklevel=2)
+        return self.errback(fail)
 
+    def armAndCallback(self, result):
+        warnings.warn("Deferred.armAndErrback is deprecated. You should be calling .callback instead.", DeprecationWarning, stacklevel=2)
+        return self.callback(result)
+
+    def armAndChain(self, d):
+        warnings.warn("Deferred.armAndChain is deprecated. You should be calling .chainDeferred instead.", DeprecationWarning, stacklevel=2)
+        return self.chainDeferred(d)
 
     def __str__(self):
         cname = self.__class__.__name__
         if hasattr(self, 'result'):
-            return "<%s at %s  current result: %r>" % (cname, hex(id(self)),
+            return "<%s at %s  current result: %r>" % (cname, hex(unsignedID(self)),
                                                        self.result)
-        return "<%s at %s>" % (cname, hex(id(self)))
+        return "<%s at %s>" % (cname, hex(unsignedID(self)))
     __repr__ = __str__
 
+class DebugInfo:
+    """Deferred debug helper"""
+    failResult = None
+
+    def _getDebugTracebacks(self):
+        info = ''
+        if hasattr(self, "creator"):
+            info += " C: Deferred was created:\n C:"
+            info += "".join(self.creator).rstrip().replace("\n","\n C:")
+            info += "\n"
+        if hasattr(self, "invoker"):
+            info += " I: First Invoker was:\n I:"
+            info += "".join(self.invoker).rstrip().replace("\n","\n I:")
+            info += "\n"
+        return info
 
     def __del__(self):
         """Print tracebacks and die.
@@ -405,13 +422,12 @@ class Deferred:
         If the *last* (and I do mean *last*) callback leaves me in an error
         state, print a traceback (if said errback is a Failure).
         """
-        if (self.called and
-            isinstance(self.result, failure.Failure)):
+        if self.failResult is not None:
             log.msg("Unhandled error in Deferred:", isError=True)
-            if self.debug:
-                log.msg("(debug: " + self._debugInfo() + ")", isError=True)
-            log.err(self.result)
-
+            debugInfo = self._getDebugTracebacks()
+            if debugInfo != '':
+                log.msg("(debug: " + debugInfo + ")", isError=True)
+            log.err(self.failResult)
 
 class DeferredList(Deferred):
     """I combine a group of deferreds into one callback.
@@ -469,7 +485,6 @@ class DeferredList(Deferred):
 
     def addDeferred(self, deferred):
         """DEPRECATED"""
-        import warnings
         warnings.warn('DeferredList.addDeferred is deprecated.',
                       DeprecationWarning, stacklevel=2)
         self.resultList.append(None)
@@ -523,7 +538,295 @@ def gatherResults(deferredList, fireOnOneErrback=0):
 SUCCESS = True
 FAILURE = False
 
+
+
+
+class waitForDeferred:
+    """
+    API Stability: semi-stable
+
+    Maintainer: U{Christopher Armstrong<mailto:radix@twistedmatrix.com>}
+
+    waitForDeferred and deferredGenerator help you write
+    Deferred-using code that looks like it's blocking (but isn't
+    really), with the help of generators.
+
+    There are two important functions involved: waitForDeferred, and
+    deferredGenerator.
+
+        def thingummy():
+            thing = waitForDeferred(makeSomeRequestResultingInDeferred())
+            yield thing
+            thing = thing.getResult()
+            print thing #the result! hoorj!
+        thingummy = deferredGenerator(thingummy)
+
+    waitForDeferred returns something that you should immediately yield;
+    when your generator is resumed, calling thing.getResult() will either
+    give you the result of the Deferred if it was a success, or raise an
+    exception if it was a failure.
+
+    deferredGenerator takes one of these waitForDeferred-using
+    generator functions and converts it into a function that returns a
+    Deferred. The result of the Deferred will be the last
+    value that your generator yielded (remember that 'return result' won't
+    work; use 'yield result; return' in place of that).
+
+    Note that not yielding anything from your generator will make the
+    Deferred result in None. Yielding a Deferred from your generator
+    is also an error condition; always yield waitForDeferred(d)
+    instead.
+
+    The Deferred returned from your deferred generator may also
+    errback if your generator raised an exception.
+
+        def thingummy():
+            thing = waitForDeferred(makeSomeRequestResultingInDeferred())
+            yield thing
+            thing = thing.getResult()
+            if thing == 'I love Twisted':
+                # will become the result of the Deferred
+                yield 'TWISTED IS GREAT!'
+                return
+            else:
+                # will trigger an errback
+                raise Exception('DESTROY ALL LIFE')
+        thingummy = deferredGenerator(thingummy)
+
+    Put succinctly, these functions connect deferred-using code with this
+    'fake blocking' style in both directions: waitForDeferred converts from
+    a Deferred to the 'blocking' style, and deferredGenerator converts from
+    the 'blocking' style to a Deferred.
+    """
+    def __init__(self, d):
+        if not isinstance(d, Deferred):
+            raise TypeError("You must give waitForDeferred a Deferred. You gave it %r." % (d,))
+        self.d = d
+
+    def getResult(self):
+        if hasattr(self, 'failure'):
+            self.failure.raiseException()
+        return self.result
+
+def _deferGenerator(g, deferred=None, result=None):
+    """
+    See L{waitForDeferred}.
+    """
+    if deferred is None:
+        deferred = Deferred()
+    try:
+        result = g.next()
+    except StopIteration:
+        deferred.callback(result)
+        return deferred
+    except:
+        deferred.errback()
+        return deferred
+
+    # Deferred.callback(Deferred) raises an error; we catch this case
+    # early here and give a nicer error message to the user in case
+    # they yield a Deferred. Perhaps eventually these semantics may
+    # change.
+    if isinstance(result, Deferred):
+        return fail(TypeError("Yield waitForDeferred(d), not d!"))
+
+    if isinstance(result, waitForDeferred):
+        def gotResult(r):
+            result.result = r
+            _deferGenerator(g, deferred, r)
+        def gotError(f):
+            result.failure = f
+            _deferGenerator(g, deferred, f)
+        result.d.addCallbacks(gotResult, gotError)
+    else:
+        _deferGenerator(g, deferred, result)
+    return deferred
+
+def deferredGenerator(f):
+    """
+    See L{waitForDeferred}.
+    """
+    return lambda *args, **kwargs: _deferGenerator(f(*args, **kwargs))
+
+
+class _ConcurrencyPrimitive(object):
+    def __init__(self):
+        self.waiting = []
+
+    def _releaseAndReturn(self, r):
+        self.release()
+        return r
+
+    def run(*args, **kwargs):
+        """Acquire, run, release.
+
+        This function takes a callable as its first argument and any
+        number of other positional and keyword arguments.  When the
+        lock or semaphore is acquired, the callable will be invoked
+        with those arguments.
+
+        @return Deferred of function result.
+        """
+        if len(args) < 2:
+            if not args:
+                raise TypeError("run() takes at least 2 arguments, none given.")
+            raise TypeError("%s.run() takes at least 2 arguments, 1 given" % (
+                args[0].__class__.__name__,))
+        self, f = args[:2]
+        args = args[2:]
+
+        def execute(ignoredResult):
+            d = maybeDeferred(f, *args, **kwargs)
+            d.addBoth(self._releaseAndReturn)
+            return d
+
+        d = self.acquire()
+        d.addCallback(execute)
+        return d
+
+
+class DeferredLock(_ConcurrencyPrimitive):
+    """A lock for event driven systems.
+
+    API stability: Unstable
+
+    @ivar locked: True when this Lock has been acquired, false at all
+    other times.  Do not change this value, but it is useful to
+    examine for the equivalent of a \"non-blocking\" acquisition.
+    """
+
+    locked = 0
+
+    def acquire(self):
+        """Attempt to acquire the lock.
+
+        @return: a Deferred which fires on lock acquisition.
+        """
+        d = Deferred()
+        if self.locked:
+            self.waiting.append(d)
+        else:
+            self.locked = 1
+            d.callback(self)
+        return d
+
+    def release(self):
+        """Release the lock.
+
+        Should be called by whomever did the acquire() when the shared
+        resource is free.
+        """
+        assert self.locked, "Tried to release an unlocked lock"
+        self.locked = 0
+        if self.waiting:
+            # someone is waiting to acquire lock
+            self.locked = 1
+            d = self.waiting.pop(0)
+            d.callback(self)
+
+class DeferredSemaphore(_ConcurrencyPrimitive):
+     """A semaphore for event driven systems.
+
+     API stability: Unstable
+     """
+
+     def __init__(self, tokens):
+         _ConcurrencyPrimitive.__init__(self)
+         self.tokens = tokens
+         self.limit = tokens
+
+     def acquire(self):
+         """Attempt to acquire the token.
+
+         @return: a Deferred which fires on token acquisition.
+         """
+         assert self.tokens >= 0, "Internal inconsistency??  tokens should never be negative"
+         d = Deferred()
+         if not self.tokens:
+             self.waiting.append(d)
+         else:
+             self.tokens = self.tokens - 1
+             d.callback(self)
+         return d
+
+     def release(self):
+         """Release the token.
+
+         Should be called by whoever did the acquire() when the shared
+         resource is free.
+         """
+         assert self.tokens < self.limit, "Someone released me too many times: too many tokens!"
+         self.tokens = self.tokens + 1
+         if self.waiting:
+             # someone is waiting to acquire token
+             self.tokens = self.tokens - 1
+             d = self.waiting.pop(0)
+             d.callback(self)
+
+class QueueOverflow(Exception):
+    pass
+
+class QueueUnderflow(Exception):
+    pass
+
+
+class DeferredQueue(object):
+     """An event driven queue.
+
+     API stability: Unstable
+
+     Objects may be added as usual to this queue.  When an attempt is
+     made to retrieve an object when the queue is empty, a Deferred is
+     returned which will fire when an object becomes available.
+
+     @ivar size: The maximum number of objects to allow into the queue
+     at a time.  When an attempt to add a new object would exceed this
+     limit, QueueOverflow is raised synchronously.  None for no limit.
+
+     @ivar backlog: The maximum number of Deferred gets to allow at
+     one time.  When an attempt is made to get an object which would
+     exceed this limit, QueueUnderflow is raised synchronously.  None
+     for no limit.
+     """
+
+     def __init__(self, size=None, backlog=None):
+         self.waiting = []
+         self.pending = []
+         self.size = size
+         self.backlog = backlog
+
+     def put(self, obj):
+         """Add an object to this queue.
+
+         @raise QueueOverflow: Too many objects are in this queue.
+         """
+         if self.waiting:
+             self.waiting.pop(0).callback(obj)
+         elif self.size is None or len(self.pending) < self.size:
+             self.pending.append(obj)
+         else:
+             raise QueueOverflow()
+
+     def get(self):
+         """Attempt to retrieve and remove an object from the queue.
+
+         @return: a Deferred which fires with the next object available in the queue.
+
+         @raise QueueUnderflow: Too many (more than C{backlog})
+         Deferreds are already waiting for an object from this queue.
+         """
+         if self.pending:
+             return succeed(self.pending.pop(0))
+         elif self.backlog is None or len(self.waiting) < self.backlog:
+             d = Deferred()
+             self.waiting.append(d)
+             return d
+         else:
+             raise QueueUnderflow()
+
+
 __all__ = ["Deferred", "DeferredList", "succeed", "fail", "FAILURE", "SUCCESS",
            "AlreadyCalledError", "TimeoutError", "gatherResults",
-           "maybeDeferred",
+           "maybeDeferred", "waitForDeferred", "deferredGenerator",
+           "DeferredLock", "DeferredSemaphore", "DeferredQueue",
           ]
