@@ -1,21 +1,9 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """
-This module provides support for Twisted to interact with the PyGTK2 mainloop.
+This module provides support for Twisted to interact with the glib/gtk2 mainloop.
 
 In order to use this support, simply do the following::
 
@@ -25,6 +13,10 @@ In order to use this support, simply do the following::
 Then use twisted.internet APIs as usual.  The other methods here are not
 intended to be called directly.
 
+When installing the reactor, you can choose whether to use the glib
+event loop or the GTK+ event loop which is based on it but adds GUI
+integration.
+
 API Stability: stable
 
 Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
@@ -33,45 +25,83 @@ Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 __all__ = ['install']
 
 # System Imports
-import sys, time
+import sys
+from zope.interface import implements
 try:
     if not hasattr(sys, 'frozen'):
         # Don't want to check this for py2exe
         import pygtk
         pygtk.require('2.0')
-except ImportError, AttributeError:
+except (ImportError, AttributeError):
     pass # maybe we're using pygtk before this hack existed.
-import gtk
+import gobject
 
 # Twisted Imports
-from twisted.python import log, threadable, runtime, failure
+from twisted.python import log, threadable, runtime, failure, components
 from twisted.internet.interfaces import IReactorFDSet
 
-# Sibling Imports
-from twisted.internet import main, default, error
+if hasattr(gobject, "threads_init"):
+    # initialize threads early, otherwise timers added before the reactor
+    # starts cause all sorts of problems
+    
+    # recent versions of python-gtk expose this. python-gtk=2.4.1
+    # (wrapping glib-2.4.7) does. python-gtk=2.0.0 (wrapping
+    # glib-2.2.3) does not.
+    gobject.threads_init()
+    threadable.init(1)
 
-reads = default.reads
-writes = default.writes
+# Sibling Imports
+from twisted.internet import main, posixbase, error, selectreactor
+
+reads = {}
+writes = {}
 hasReader = reads.has_key
 hasWriter = writes.has_key
 
 # the next callback
 _simtag = None
-POLL_DISCONNECTED = gtk._gobject.IO_HUP | gtk._gobject.IO_ERR | \
-                    gtk._gobject.IO_NVAL
+POLL_DISCONNECTED = gobject.IO_HUP | gobject.IO_ERR | gobject.IO_NVAL
 
-# gtk's iochannel sources won't tell us about any events that we haven't
+# glib's iochannel sources won't tell us about any events that we haven't
 # asked for, even if those events aren't sensible inputs to the poll()
 # call.
-INFLAGS = gtk._gobject.IO_IN | POLL_DISCONNECTED
-OUTFLAGS = gtk._gobject.IO_OUT | POLL_DISCONNECTED
+INFLAGS = gobject.IO_IN | POLL_DISCONNECTED
+OUTFLAGS = gobject.IO_OUT | POLL_DISCONNECTED
 
+def _our_mainquit():
+    # XXX: gtk.main_quit() (which is used for crash()) raises an exception if
+    # gtk.main_level() == 0; however, all the tests freeze if we use this
+    # function to stop the reactor.  what gives?  (I believe this may have been
+    # a stupid mistake where I forgot to import gtk here... I will remove this
+    # comment if the tests pass)
+    import gtk
+    if gtk.main_level():
+        gtk.main_quit()
 
-class Gtk2Reactor(default.PosixReactorBase):
+class Gtk2Reactor(posixbase.PosixReactorBase):
     """GTK+-2 event loop reactor.
     """
 
-    __implements__ = (default.PosixReactorBase.__implements__, IReactorFDSet)
+    implements(IReactorFDSet)
+
+    def __init__(self, useGtk=True):
+        self.context = gobject.main_context_default()
+        self.loop = gobject.MainLoop()
+        posixbase.PosixReactorBase.__init__(self)
+        # pre 2.3.91 the glib iteration and mainloop functions didn't release
+        # global interpreter lock, thus breaking thread and signal support.
+        if (hasattr(gobject, "pygtk_version") and gobject.pygtk_version >= (2, 3, 91)
+            and not useGtk):
+            self.__pending = self.context.pending
+            self.__iteration = self.context.iteration
+            self.__crash = self.loop.quit
+            self.__run = self.loop.run
+        else:
+            import gtk
+            self.__pending = gtk.events_pending
+            self.__iteration = gtk.main_iteration
+            self.__crash = _our_mainquit
+            self.__run = gtk.main
 
     # The input_add function in pygtk1 checks for objects with a
     # 'fileno' method and, if present, uses the result of that method
@@ -87,10 +117,9 @@ class Gtk2Reactor(default.PosixReactorBase):
             # handle python objects
             def wrapper(source, condition, real_s=source, real_cb=callback):
                 return real_cb(real_s, condition)
-            return gtk._gobject.io_add_watch(source.fileno(), condition,
-                                             wrapper)
+            return gobject.io_add_watch(source.fileno(), condition, wrapper)
         else:
-            return gtk._gobject.io_add_watch(source, condition, callback)
+            return gobject.io_add_watch(source, condition, callback)
 
     def addReader(self, reader):
         if not hasReader(reader):
@@ -101,19 +130,16 @@ class Gtk2Reactor(default.PosixReactorBase):
             writes[writer] = self.input_add(writer, OUTFLAGS, self.callback)
 
     def removeAll(self):
-        v = reads.keys()
-        for reader in v:
-            self.removeReader(reader)
-        return v
-
+        return self._removeAll(reads, writes)
+    
     def removeReader(self, reader):
         if hasReader(reader):
-            gtk.input_remove(reads[reader])
+            gobject.source_remove(reads[reader])
             del reads[reader]
 
     def removeWriter(self, writer):
         if hasWriter(writer):
-            gtk.input_remove(writes[writer])
+            gobject.source_remove(writes[writer])
             del writes[writer]
 
     doIterationTimer = None
@@ -121,70 +147,66 @@ class Gtk2Reactor(default.PosixReactorBase):
     def doIterationTimeout(self, *args):
         self.doIterationTimer = None
         return 0 # auto-remove
+    
     def doIteration(self, delay):
         # flush some pending events, return if there was something to do
-        # don't use the usual "while gtk.events_pending(): mainiteration()"
+        # don't use the usual "while self.context.pending(): self.context.iteration()"
         # idiom because lots of IO (in particular test_tcp's
         # ProperlyCloseFilesTestCase) can keep us from ever exiting.
         log.msg(channel='system', event='iteration', reactor=self)
-        if gtk.events_pending():
-            gtk.main_iteration(0)
+        if self.__pending():
+            self.__iteration(0)
             return
         # nothing to do, must delay
         if delay == 0:
             return # shouldn't delay, so just return
-        self.doIterationTimer = gtk.timeout_add(int(delay * 1000),
+        self.doIterationTimer = gobject.timeout_add(int(delay * 1000),
                                                 self.doIterationTimeout)
         # This will either wake up from IO or from a timeout.
-        gtk.main_iteration(1) # block
+        self.__iteration(1) # block
         # note: with the .simulate timer below, delays > 0.1 will always be
         # woken up by the .simulate timer
         if self.doIterationTimer:
             # if woken by IO, need to cancel the timer
-            gtk.timeout_remove(self.doIterationTimer)
+            gobject.source_remove(self.doIterationTimer)
             self.doIterationTimer = None
 
     def crash(self):
-        gtk.main_quit()
+        self.__crash()
 
     def run(self, installSignalHandlers=1):
         self.startRunning(installSignalHandlers=installSignalHandlers)
-        self.simulate()
-        gtk.main()
-
+        gobject.timeout_add(0, self.simulate)
+        self.__run()
+    
     def _doReadOrWrite(self, source, condition, faildict={
         error.ConnectionDone: failure.Failure(error.ConnectionDone()),
-        error.ConnectionLost: failure.Failure(error.ConnectionLost())  }):
+        error.ConnectionLost: failure.Failure(error.ConnectionLost()),
+        }):
         why = None
+        didRead = None
         if condition & POLL_DISCONNECTED and \
-               not (condition & gtk._gobject.IO_IN):
+               not (condition & gobject.IO_IN):
             why = main.CONNECTION_LOST
         else:
             try:
-                didRead = None
-                if condition & gtk._gobject.IO_IN:
+                if condition & gobject.IO_IN:
                     why = source.doRead()
                     didRead = source.doRead
-                if not why and condition & gtk._gobject.IO_OUT:
+                if not why and condition & gobject.IO_OUT:
                     # if doRead caused connectionLost, don't call doWrite
                     # if doRead is doWrite, don't call it again.
                     if not source.disconnected and source.doWrite != didRead:
                         why = source.doWrite()
+                        didRead = source.doWrite # if failed it was in write
             except:
                 why = sys.exc_info()[1]
                 log.msg('Error In %s' % source)
                 log.deferr()
 
         if why:
-            self.removeReader(source)
-            self.removeWriter(source)
-            f = faildict.get(why.__class__)
-            if f:
-                source.connectionLost(f)
-            else:
-                source.connectionLost(failure.Failure(why))
-
-
+            self._disconnectSelectable(source, why, didRead == source.doRead)
+    
     def callback(self, source, condition):
         log.callWithLogger(source, self._doReadOrWrite, source, condition)
         self.simulate() # fire Twisted timers
@@ -195,25 +217,29 @@ class Gtk2Reactor(default.PosixReactorBase):
         """
         global _simtag
         if _simtag is not None:
-            gtk.timeout_remove(_simtag)
+            gobject.source_remove(_simtag)
         self.runUntilCurrent()
         timeout = min(self.timeout(), 0.1)
         if timeout is None:
             timeout = 0.1
         # grumble
-        _simtag = gtk.timeout_add(int(timeout * 1010), self.simulate)
+        _simtag = gobject.timeout_add(int(timeout * 1010), self.simulate)
+
+components.backwardsCompatImplements(Gtk2Reactor)
 
 
-class PortableGtkReactor(default.SelectReactor):
+class PortableGtkReactor(selectreactor.SelectReactor):
     """Reactor that works on Windows.
 
     input_add is not supported on GTK+ for Win32, apparently.
     """
 
     def crash(self):
+        import gtk
         gtk.mainquit()
 
     def run(self, installSignalHandlers=1):
+        import gtk
         self.startRunning(installSignalHandlers=installSignalHandlers)
         self.simulate()
         gtk.mainloop()
@@ -223,24 +249,27 @@ class PortableGtkReactor(default.SelectReactor):
         """
         global _simtag
         if _simtag is not None:
-            gtk.timeout_remove(_simtag)
+            gobject.source_remove(_simtag)
         self.iterate()
         timeout = min(self.timeout(), 0.1)
         if timeout is None:
             timeout = 0.1
         # grumble
-        _simtag = gtk.timeout_add(int(timeout * 1010), self.simulate)
+        _simtag = gobject.timeout_add(int(timeout * 1010), self.simulate)
 
 
-def install():
+def install(useGtk=True):
     """Configure the twisted mainloop to be run inside the gtk mainloop.
+
+    @param useGtk: should glib rather than GTK+ event loop be
+    used (this will be slightly faster but does not support GUI).
     """
-    reactor = Gtk2Reactor()
+    reactor = Gtk2Reactor(useGtk)
     from twisted.internet.main import installReactor
     installReactor(reactor)
     return reactor
 
-def portableInstall():
+def portableInstall(useGtk=True):
     """Configure the twisted mainloop to be run inside the gtk mainloop.
     """
     reactor = PortableGtkReactor()
