@@ -1,20 +1,8 @@
 # -*- test-case-name: twisted.test.test_process -*-
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """UNIX Process management.
 
@@ -40,11 +28,13 @@ from twisted.persisted import styles
 from twisted.python import log, failure
 from twisted.python.util import switchUID
 from twisted.internet import protocol
+from twisted.internet.error import ProcessExitedAlready
 
 # Sibling Imports
-import abstract, main, fdesc, error
+import abstract, fdesc, error
 from main import CONNECTION_LOST, CONNECTION_DONE
 
+
 reapProcessHandlers = {}
 
 def reapAllProcesses():
@@ -123,8 +113,21 @@ class ProcessWriter(abstract.FileDescriptor):
         abstract.FileDescriptor.write(self, data)
 
     def doRead(self):
-        """The only way this pipe can become readable is at EOF, because the
-        child has closed it.
+        """The only way a write pipe can become "readable" is at EOF, because the
+        child has closed it, and we're using a reactor which doesn't distinguish
+        between readable and closed (such as the select reactor).
+        
+        Except that's not true on linux < 2.6.11. It has the following characteristics:
+        write pipe is completely empty => POLLOUT (writable in select)
+        write pipe is not completely empty => POLLIN (readable in select)
+        write pipe's reader closed => POLLIN|POLLERR (readable and writable in select)
+        
+        That's what this funky code is for. If linux was not broken, this function could
+        be simply "return CONNECTION_LOST".
+        
+        BUG: We call select no matter what the reactor.
+        If the reactor is pollreactor, and the fd is > 1024, this will fail.
+        (only occurs on broken versions of linux, though).
         """
         fd = self.fd
         r, w, x = select.select([fd], [fd], [], 0)
@@ -136,7 +139,7 @@ class ProcessWriter(abstract.FileDescriptor):
         """
         abstract.FileDescriptor.connectionLost(self, reason)
         os.close(self.fd)
-        self.proc.childConnectionLost(self.name)
+        self.proc.childConnectionLost(self.name, reason)
 
 
 class ProcessReader(abstract.FileDescriptor):
@@ -145,6 +148,8 @@ class ProcessReader(abstract.FileDescriptor):
     I am a selectable representation of a process's output pipe, such as
     stdout and stderr.
     """
+    connected = 1
+
     def __init__(self, reactor, proc, name, fileno):
         """Initialize, specifying a process to connect to.
         """
@@ -173,18 +178,19 @@ class ProcessReader(abstract.FileDescriptor):
     def dataReceived(self, data):
         self.proto.childDataReceived(self.name, data)
 
+    def loseConnection(self):
+        if self.connected and not self.disconnecting:
+            self.disconnecting = 1
+            self.stopReading()
+            self.reactor.callLater(0, self.connectionLost, failure.Failure(CONNECTION_DONE))
+    
     def connectionLost(self, reason):
         """Close my end of the pipe, signal the Process (which signals the
         ProcessProtocol).
         """
         abstract.FileDescriptor.connectionLost(self, reason)
         os.close(self.fd)
-        self.proc.childConnectionLost(self.name)
-
-
-class ProcessExitedAlready(Exception):
-    """The process has already excited, and the operation requested can no longer be performed."""
-    pass
+        self.proc.childConnectionLost(self.name, reason)
 
 
 class Process(styles.Ephemeral):
@@ -237,7 +243,18 @@ class Process(styles.Ephemeral):
         """
 
         if not proto:
-            assert 'r' not in childFDs.values() and 'w' not in childFDs.values()
+            assert 'r' not in childFDs.values()
+            assert 'w' not in childFDs.values()
+        if not signal.getsignal(signal.SIGCHLD):
+            log.msg("spawnProcess called, but the SIGCHLD handler is not " +
+                    "installed. This probably means you have not yet " +
+                    "called reactor.run, or called " + 
+                    "reactor.run(installSignalHandler=0). You will probably " +
+                    "never see this process finish, and it may become a " +
+                    "zombie process.")
+            # if you see this message during a unit test, look in
+            # test-standard.xhtml or twisted.test.test_process.SignalMixin
+            # for a workaround
 
         self.lostProcess = False
 
@@ -540,8 +557,11 @@ class Process(styles.Ephemeral):
         self.closeStdout()
     def write(self,data):
         """Call this to write to standard input on this process.
+        
+        NOTE: This will silently lose data if there is no standard input.
         """
-        self.pipes[0].write(data)
+        if self.pipes.has_key(0):
+            self.pipes[0].write(data)
 
     def signalProcess(self, signalID):
         if signalID in ('HUP', 'STOP', 'INT', 'KILL', 'TERM'):
@@ -560,7 +580,7 @@ class Process(styles.Ephemeral):
         ##self.closeStdin()
         self.maybeCallProcessEnded()
 
-    def childConnectionLost(self, childFD):
+    def childConnectionLost(self, childFD, reason):
         # this is called when one of the helpers (ProcessReader or
         # ProcessWriter) notices their pipe has been closed
         del self.pipes[childFD]
@@ -648,15 +668,26 @@ class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
             try:
                 sys.settrace(None)
                 os.close(masterfd)
+                if hasattr(termios, 'TIOCNOTTY'):
+                    try:
+                        fd = os.open("/dev/tty", os.O_RDWR | os.O_NOCTTY)
+                    except OSError:
+                        pass
+                    else:
+                        try:
+                            fcntl.ioctl(fd, termios.TIOCNOTTY, '')
+                        except:
+                            pass
+                        os.close(fd)
+                    
                 os.setsid()
+                
                 if hasattr(termios, 'TIOCSCTTY'):
                     fcntl.ioctl(slavefd, termios.TIOCSCTTY, '')
-                else:
-                    for fd in range(3):
-                        if fd != slavefd:
-                            os.close(fd)
-                    fd = os.open(ttyname, os.O_RDWR)
-                    os.close(fd)
+                
+                for fd in range(3):
+                    if fd != slavefd:
+                        os.close(fd)
 
                 os.dup2(slavefd, 0) # stdin
                 os.dup2(slavefd, 1) # stdout
