@@ -1,20 +1,8 @@
 # -*- test-case-name: twisted.test.test_unix -*-
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """Various asynchronous TCP/IP classes.
 
@@ -26,6 +14,7 @@ Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 # System imports
 import os, stat, socket
 from errno import *
+from zope.interface import implements, implementsOnly, implementedBy
 
 if not hasattr(socket, 'AF_UNIX'):
     raise ImportError, "UNIX sockets not supported on this platform"
@@ -33,7 +22,7 @@ if not hasattr(socket, 'AF_UNIX'):
 # Twisted imports
 from twisted.internet import base, tcp, udp, error, interfaces, protocol, address, defer
 from twisted.internet.error import CannotListenError
-from twisted.python import lockfile, log, reflect, failure
+from twisted.python import lockfile, log, reflect, failure, components
 
 
 class Server(tcp.Server):
@@ -157,7 +146,7 @@ class Connector(base.BaseConnector):
 class DatagramPort(udp.Port):
     """Datagram UNIX port, listening for packets."""
 
-    __implements__ = base.BasePort.__implements__, interfaces.IUNIXDatagramTransport
+    implements(interfaces.IUNIXDatagramTransport)
 
     addressFamily = socket.AF_UNIX
 
@@ -222,11 +211,14 @@ class DatagramPort(udp.Port):
     def getHost(self):
         return address.UNIXAddress(self.socket.getsockname())
 
+components.backwardsCompatImplements(DatagramPort)
+
 
 class ConnectedDatagramPort(DatagramPort):
     """A connected datagram UNIX socket."""
 
-    __implements__ = base.BasePort.__implements__,  interfaces.IUNIXDatagramConnectedTransport
+    implementsOnly(interfaces.IUNIXDatagramConnectedTransport,
+                   *(implementedBy(base.BasePort)))
 
     def __init__(self, addr, proto, maxPacketSize=8192, mode=0666, bindAddress=None, reactor=None):
         assert isinstance(proto, protocol.ConnectedDatagramProtocol)
@@ -288,6 +280,9 @@ class ConnectedDatagramPort(DatagramPort):
     def getPeer(self):
         return address.UNIXAddress(self.remoteaddr)
 
+components.backwardsCompatImplements(ConnectedDatagramPort)
+
+
 def _checkPIDFile(pidname):
     try:
         pid = int(open(pidfile).read())
