@@ -1,18 +1,6 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """A win32event based implementation of the Twisted main loop.
 
@@ -43,7 +31,7 @@ ALTERNATIVE SOLUTIONS:
 
 Or:
 
- - Instead of doing a reactor, we make this an addon to the default reactor.
+ - Instead of doing a reactor, we make this an addon to the select reactor.
    The WFMO event loop runs in a separate thread. This means no need to maintain
    separate code for networking, 64 event limit doesn't apply to sockets,
    we can run processes and other win32 stuff in default event loop. The
@@ -71,9 +59,9 @@ import msvcrt
 import win32gui
 
 # Twisted imports
-from twisted.internet import abstract, default, main, error
-from twisted.python import log, threadable, failure
-from twisted.internet.interfaces import IReactorFDSet
+from twisted.internet import abstract, posixbase, main, error
+from twisted.python import log, threadable, failure, components
+from twisted.internet.interfaces import IReactorFDSet, IReactorProcess
 
 # System imports
 import os
@@ -82,6 +70,7 @@ import Queue
 import string
 import time
 import sys
+from zope.interface import implements
 
 
 # globals
@@ -90,10 +79,10 @@ writes = {}
 events = {}
 
 
-class Win32Reactor(default.PosixReactorBase):
+class Win32Reactor(posixbase.PosixReactorBase):
     """Reactor that uses Win32 event APIs."""
 
-    __implements__ = (default.PosixReactorBase.__implements__, IReactorFDSet)
+    implements(IReactorFDSet, IReactorProcess)
 
     dummyEvent = CreateEvent(None, 0, 0, None)
 
@@ -116,7 +105,7 @@ class Win32Reactor(default.PosixReactorBase):
         """Add a socket FileDescriptor for notification of data available to read.
         """
         if not reads.has_key(reader):
-            reads[reader] = self._makeSocketEvent(reader, reader.doRead, FD_READ|FD_ACCEPT|FD_CONNECT|FD_CLOSE)
+            reads[reader] = self._makeSocketEvent(reader, 'doRead', FD_READ|FD_ACCEPT|FD_CONNECT|FD_CLOSE)
 
     def addWriter(self, writer, writes=writes):
         """Add a socket FileDescriptor for notification of data available to write.
@@ -139,11 +128,7 @@ class Win32Reactor(default.PosixReactorBase):
 
     def removeAll(self):
         """Remove all selectables, and return a list of them."""
-        result = reads.keys() + writes.keys()
-        reads.clear()
-        writes.clear()
-        events.clear()
-        return result
+        return self._removeAll(reads, writes)
 
     def doWaitForMultipleEvents(self, timeout,
                                 reads=reads,
@@ -201,19 +186,14 @@ class Win32Reactor(default.PosixReactorBase):
             return 1
 
     def _runAction(self, action, fd):
-            try:
-                closed = action()
-            except:
-                closed = sys.exc_info()[1]
-                log.deferr()
+        try:
+            closed = getattr(fd, action)()
+        except:
+            closed = sys.exc_info()[1]
+            log.deferr()
 
-            if closed:
-                self.removeReader(fd)
-                self.removeWriter(fd)
-                try:
-                    fd.connectionLost(failure.Failure(closed))
-                except:
-                    log.deferr()
+        if closed:
+            self._disconnectSelectable(fd, closed, action == 'doRead')
 
     doIteration = doWaitForMultipleEvents
 
@@ -221,6 +201,8 @@ class Win32Reactor(default.PosixReactorBase):
         """Spawn a process."""
         Process(self, processProtocol, executable, args, env, path)
 
+components.backwardsCompatImplements(Win32Reactor)
+
 
 def install():
     threadable.init(1)
@@ -291,9 +273,15 @@ class Process(abstract.FileDescriptor):
         win32file.CloseHandle(self.hStdinW)
         self.hStdinW = tmp
 
+        # Add the specified environment to the current environment - this is
+        # necessary because certain operations are only supported on Windows
+        # if certain environment variables are present.
+        env = os.environ.copy()
+        env.update(environment or {})
+
         # create the process
         cmdline = "%s %s" % (command, string.join(args[1:], ' '))
-        self.hProcess, hThread, dwPid, dwTid = win32process.CreateProcess(None, cmdline, None, None, 1, 0, environment, path, StartupInfo)
+        self.hProcess, hThread, dwPid, dwTid = win32process.CreateProcess(None, cmdline, None, None, 1, 0, env, path, StartupInfo)
 
         # close handles which only the child will use
         win32file.CloseHandle(hStderrW)
@@ -307,7 +295,7 @@ class Process(abstract.FileDescriptor):
         # notify protocol
         self.protocol.makeConnection(self)
 
-        self.reactor.addEvent(self.hProcess, self, self.inConnectionLost)
+        self.reactor.addEvent(self.hProcess, self, 'inConnectionLost')
         threading.Thread(target=self.doWrite).start()
         threading.Thread(target=self.doReadOut).start()
         threading.Thread(target=self.doReadErr).start()
