@@ -1,19 +1,7 @@
-# -*- test-case-name: twisted.test.test_names -*-
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# -*- test-case-name: twisted.names.test.test_names -*-
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 from __future__ import nested_scopes
 
@@ -22,7 +10,7 @@ import copy
 import operator
 import time
 
-from twisted.protocols import dns
+from twisted.names import dns
 from twisted.internet import defer
 from twisted.python import failure
 
@@ -114,7 +102,7 @@ class FileAuthority(common.ResolverBase):
                     )
             if not results:
                 results = cnames
-            
+
             for record in results + authority:
                 if record.type == dns.NS or record.type == dns.CNAME:
                     n = str(record.payload.name)
@@ -123,6 +111,7 @@ class FileAuthority(common.ResolverBase):
                             additional.append(
                                 dns.RRHeader(n, dns.A, dns.IN, rec.ttl or default_ttl, rec, auth=True)
                             )
+            
             return defer.succeed((results, authority, additional))
         except KeyError:
             if name.lower().endswith(self.soa[0].lower()):
