@@ -1,18 +1,7 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# -*- test-case-name: twisted.names.test.test_names -*-
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """
 Asynchronous client DNS
@@ -35,13 +24,13 @@ import time
 # Twisted imports
 from twisted.python.runtime import platform
 from twisted.internet import defer, protocol, interfaces, threads
-from twisted.python import log, failure
-from twisted.protocols import dns
-
+from twisted.python import log, failure, components
+from twisted.names import dns
+from zope.interface import implements
 import common
 
 class Resolver(common.ResolverBase):
-    __implements__ = (interfaces.IResolver,)
+    implements(interfaces.IResolver)
 
     index = 0
     timeout = None
@@ -121,7 +110,8 @@ class Resolver(common.ResolverBase):
             resolvConf = file(self.resolv)
         except IOError, e:
             if e.errno == errno.ENOENT:
-                pass
+                # Missing resolv.conf is treated the same as an empty resolv.conf 
+                self.parseConfig(())
             else:
                 raise
         else:
@@ -131,7 +121,7 @@ class Resolver(common.ResolverBase):
                 self._lastResolvTime = mtime
                 self.parseConfig(resolvConf)
 
-        # Check the mtime again in a little while
+        # Check again in a little while
         from twisted.internet import reactor
         self._parseCall = reactor.callLater(self._resolvReadInterval, self.maybeParseConfig)
 
@@ -156,6 +146,8 @@ class Resolver(common.ResolverBase):
                 except IndexError:
                     self.search = ''
                 self.domain = None
+        if not servers:
+            servers.append(('127.0.0.1', dns.PORT))
         self.dynServers = servers
 
 
@@ -207,27 +199,46 @@ class Resolver(common.ResolverBase):
         if timeout is None:
             timeout = self.timeout
 
-        address = self.pickServer()
-        if address is None:
+        addresses = self.servers + list(self.dynServers)
+        if not addresses:
             return defer.fail(IOError("No domain name servers available"))
 
-        return self.protocol.query(address, queries, timeout[0]
-            ).addErrback(self._reissue, address, queries, timeout[1:]
+        used = addresses.pop()
+        return self.protocol.query(used, queries, timeout[0]
+            ).addErrback(self._reissue, addresses, [used], queries, timeout
             )
 
 
-    def _reissue(self, reason, address, query, timeout):
-        reason.trap(defer.TimeoutError)
-        if timeout and self.protocol.transport:
-            d = self.protocol.query(address, query, timeout[0], reason.value.id)
-            d.addErrback(self._reissue, address, query, timeout[1:])
-            return d
+    def _reissue(self, reason, addressesLeft, addressesUsed, query, timeout):
+        reason.trap(dns.DNSQueryTimeoutError)
 
-        try:
-            del self.protocol.resends[reason.value.id]
-        except:
-            pass
-        return failure.Failure(defer.TimeoutError(query))
+        # If there are no servers left to be tried, adjust the timeout
+        # to the next longest timeout period and move all the
+        # "used" addresses back to the list of addresses to try.
+        if not addressesLeft:
+            addressesLeft = addressesUsed
+            addressesLeft.reverse()
+            addressesUsed = []
+            timeout = timeout[1:]
+
+        # If all timeout values have been used, or the protocol has no
+        # transport, this query has failed.  Tell the protocol we're
+        # giving up on it and return a terminal timeout failure to our
+        # caller.
+        if not timeout or self.protocol.transport is None:
+            self.protocol.removeResend(reason.value.id)
+            return failure.Failure(defer.TimeoutError(query))
+
+        # Get an address to try.  Take it out of the list of addresses
+        # to try and put it ino the list of already tried addresses.
+        address = addressesLeft.pop()
+        addressesUsed.append(address)
+
+        # Issue a query to a server.  Use the current timeout.  Add this
+        # function as a timeout errback in case another retry is required.
+        d = self.protocol.query(address, query, timeout[0], reason.value.id)
+        d.addErrback(self._reissue, addressesLeft, addressesUsed, query, timeout)
+        return d
 
 
     def queryTCP(self, queries, timeout = 10):
@@ -285,8 +296,14 @@ class Resolver(common.ResolverBase):
         controller = AXFRController(name, d)
         factory = DNSClientFactory(controller, timeout)
         factory.noisy = False #stfu
-        reactor.connectTCP(host, port, factory)
-        return d.addCallback(lambda x: (x, [], []))
+        connector = reactor.connectTCP(host, port, factory)
+        return d.addCallback(self._cbLookupZone, connector)
+
+    def _cbLookupZone(self, result, connector):
+        connector.disconnect()
+        return (result, [], [])
+
+components.backwardsCompatImplements(Resolver)
 
 
 class AXFRController:
@@ -321,18 +338,7 @@ class AXFRController:
             self.deferred.callback(self.records)
 
 
-class ThreadedResolver:
-    __implements__ = (interfaces.IResolverSimple,)
-
-    def __init__(self):
-        self.cache = {}
-
-    def getHostByName(self, name, timeout = 10):
-        # XXX - Make this respect timeout
-        d = threads.deferToThread(socket.gethostbyname, name)
-        d.setTimeout(timeout)
-        return d
-
+from twisted.internet.base import ThreadedResolver
 
 class DNSClientFactory(protocol.ClientFactory):
     def __init__(self, controller, timeout = 10):
