@@ -1,19 +1,7 @@
-# -*- test-case-name: twisted.test.test_names -*-
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# -*- test-case-name: twisted.names.test.test_names -*-
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """
 Async DNS server
@@ -38,7 +26,7 @@ import time
 
 # Twisted imports
 from twisted.internet import protocol, defer
-from twisted.protocols import dns
+from twisted.names import dns
 from twisted.python import failure, log
 
 import resolve, common
