@@ -1,31 +1,15 @@
 
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
 
-#Don't use twistd on windows
-from twisted.python import runtime
-import sys
-if runtime.platformType != 'posix':
-    sys.exit("Please use twistw on windows, not twistd")
-# End hack
 
 from twisted.python import log, syslog
 from twisted.python.util import switchUID
 from twisted.application import app, service
+from twisted.scripts import mktap
 from twisted import copyright
-import os, errno, signal
+
+import os, errno, sys
 
 class ServerOptions(app.ServerOptions):
     synopsis = "Usage: twistd [options]"
@@ -50,7 +34,15 @@ class ServerOptions(app.ServerOptions):
                       "Name of the pidfile"],
                      ['chroot', None, None,
                       'Chroot to a supplied directory before running'],
+                     ['uid', 'u', None, "The uid to run as."],
+                     ['gid', 'g', None, "The gid to run as."],
                     ]
+    zsh_altArgDescr = {"prefix":"Use the given prefix when syslogging (default: twisted)",
+                       "pidfile":"Name of the pidfile (default: twistd.pid)",}
+    #zsh_multiUse = ["foo", "bar"]
+    #zsh_mutuallyExclusive = [("foo", "bar"), ("bar", "baz")]
+    zsh_actions = {"pidfile":'_files -g "*.pid"', "chroot":'_dirs'}
+    zsh_actionDescr = {"chroot":"chroot directory"}
 
     def opt_version(self):
         """Print version information and exit.
@@ -109,31 +101,43 @@ def startLogging(logfilename, sysLog, prefix, nodaemon):
         logFile = sys.stdout
     else:
         logFile = app.getLogFile(logfilename or 'twistd.log')
-        def rotateLog(signal, frame):
-            from twisted.internet import reactor
-            reactor.callLater(0, logFile.rotate)
-        signal.signal(signal.SIGUSR1, rotateLog)
+        try:
+            import signal
+        except ImportError:
+            pass
+        else:
+            def rotateLog(signal, frame):
+                from twisted.internet import reactor
+                reactor.callFromThread(logFile.rotate)
+            signal.signal(signal.SIGUSR1, rotateLog)
+        
     if not sysLog:
         log.startLogging(logFile)
     sys.stdout.flush()
 
 
 def daemonize():
+    # See http://www.erlenstar.demon.co.uk/unix/faq_toc.html#TOC16
     if os.fork():   # launch child and...
         os._exit(0) # kill off parent
     os.setsid()
+    if os.fork():   # launch child and...
+        os._exit(0) # kill off parent again.
     os.umask(077)
+    null=os.open('/dev/null', os.O_RDWR)
     for i in range(3):
         try:
-            os.close(i)
+            os.dup2(null, i)
         except OSError, e:
             if e.errno != errno.EBADF:
                 raise
+    os.close(null)
 
 def shedPrivileges(euid, uid, gid):
-    switchUID(uid, gid, euid)
-    extra = euid and 'e' or ''
-    log.msg('set %suid/%sgid %s/%s' % (extra, extra, uid, gid))
+    if uid is not None or gid is not None:
+        switchUID(uid, gid, euid)
+        extra = euid and 'e' or ''
+        log.msg('set %suid/%sgid %s/%s' % (extra, extra, uid, gid))
 
 def launchWithName(name):
     if name and name != sys.argv[0]:
@@ -157,7 +161,14 @@ def startApplication(config, application):
         launchWithName(process.processName)
     setupEnvironment(config)
     service.IService(application).privilegedStartService()
-    shedPrivileges(config['euid'], process.uid, process.gid)
+
+    uid, gid = mktap.getid(config['uid'], config['gid'])
+    if uid is None:
+        uid = process.uid
+    if gid is None:
+        gid = process.gid
+
+    shedPrivileges(config['euid'], uid, gid)
     app.startApplication(application, not config['no_save'])
 
 
@@ -184,5 +195,3 @@ def run():
     app.run(runApp, ServerOptions)
 
 
-if __name__ == "__main__":
-     run()
