@@ -1,20 +1,8 @@
 # -*- test-case-name: twisted.test.test_jelly -*-
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """S-expression-based persistence of python objects.
 
@@ -92,6 +80,16 @@ from types import NoneType
 from types import ClassType
 import copy
 
+try:
+    import datetime
+except ImportError:
+    class datetime:
+        def sorry(self, *args, **kw):
+            raise NotImplementedError("Datetime unserializing not supported in Python < 2.3")
+        datetime = sorry
+        timedelta = sorry
+        date = sorry
+        time = sorry
 try:
     from types import BooleanType
 except ImportError:
@@ -99,12 +97,12 @@ except ImportError:
 
 from new import instance
 from new import instancemethod
-
+from zope.interface import implements
 
 # Twisted Imports
 from twisted.python.reflect import namedObject, namedModule, qual
 from twisted.persisted.crefutil import NotKnown, _Tuple, _InstanceMethod, _DictKeyAndValue, _Dereference
-from twisted.python import runtime
+from twisted.python import runtime, components
 
 from twisted.spread.interfaces import IJellyable, IUnjellyable
 
@@ -274,7 +272,7 @@ class Jellyable:
     convenience method.
     """
 
-    __implements__ = IJellyable,
+    implements(IJellyable)
     
     def getStateFor(self, jellier):
         return self.__dict__
@@ -286,13 +284,16 @@ class Jellyable:
             jellier.jelly(self.getStateFor(jellier))])
         return jellier.preserve(self, sxp)
 
+components.backwardsCompatImplements(Jellyable)
+
+
 class Unjellyable:
     """
     Inherit from me to Unjelly yourself directly with the
     `setStateFor' convenience method.
     """
 
-    __implements__ = IUnjellyable,
+    implements(IUnjellyable)
     
     def setStateFor(self, unjellier, state):
         self.__dict__ = state
@@ -301,6 +302,8 @@ class Unjellyable:
         state = unjellier.unjelly(jellyList[1])
         self.setStateFor(unjellier, state)
 
+components.backwardsCompatImplements(Unjellyable)
+
 
 class _Jellier:
     """(Internal) This class manages state for a call to jelly()
@@ -433,6 +436,18 @@ class _Jellier:
                 return ['module', obj.__name__]
             elif objType is BooleanType:
                 return ['boolean', obj and 'true' or 'false']
+            elif objType is datetime.datetime:
+                if obj.tzinfo:
+                    raise NotImplementedError, "Currently can't jelly datetime objects with tzinfo"
+                return ['datetime', '%s %s %s %s %s %s %s' % (obj.year, obj.month, obj.day, obj.hour, obj.minute, obj.second, obj.microsecond)]
+            elif objType is datetime.time:
+                if obj.tzinfo:
+                    raise NotImplementedError, "Currently can't jelly datetime objects with tzinfo"
+                return ['time', '%s %s %s %s' % (obj.hour, obj.minute, obj.second, obj.microsecond)]
+            elif objType is datetime.date:
+                return ['date', '%s %s %s' % (obj.year, obj.month, obj.day)]
+            elif objType is datetime.timedelta:
+                return ['timedelta', '%s %s %s' % (obj.days, obj.seconds, obj.microseconds)]
             elif objType is ClassType or issubclass(type, objType):
                 return ['class', qual(obj)]
             else:
@@ -571,6 +586,19 @@ class _Unjellier:
         else:
             return Unpersistable(exp[0])
 
+    def _unjelly_datetime(self, exp):
+        return datetime.datetime(*map(int, exp[0].split()))
+
+    def _unjelly_date(self, exp):
+        return datetime.date(*map(int, exp[0].split()))
+
+    def _unjelly_time(self, exp):
+        return datetime.time(*map(int, exp[0].split()))
+
+    def _unjelly_timedelta(self, exp):
+        days, seconds, microseconds = map(int, exp[0].split())
+        return datetime.timedelta(days=days, seconds=seconds, microseconds=microseconds)
+
     def unjellyInto(self, obj, loc, jel):
         o = self.unjelly(jel)
         if isinstance(o, NotKnown):
@@ -770,6 +798,10 @@ class SecurityOptions:
                              "str": 1,
                              "int": 1,
                              "float": 1,
+                             "datetime": 1,
+                             "time": 1,
+                             "date": 1,
+                             "timedelta": 1,
                              "NoneType": 1}
         if hasattr(types, 'UnicodeType'):
             self.allowedTypes['unicode'] = 1
