@@ -1,19 +1,7 @@
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 from twisted.trial import unittest
 
@@ -133,10 +121,31 @@ class BananaTestCase(unittest.TestCase):
             self.enc.dataReceived(crashString)
         except banana.BananaError:
             pass
+
+    def testCrashNegativeLong(self):
+        # There was a bug in cBanana which relied on negating a negative integer
+        # always giving a postive result, but for the lowest possible number in
+        # 2s-complement arithmetic, that's not true, i.e.
+        #     long x = -2147483648;
+        #     long y = -x;
+        #     x == y;  /* true! */
+        # (assuming 32-bit longs)
+        self.enc.sendEncoded(-2147483648)
+        self.enc.dataReceived(self.io.getvalue())
+        assert self.result == -2147483648, "should be -2147483648, got %s" % self.result
+
             
 testCases = [MathTestCase, BananaTestCase]
 
-if hasattr(banana, 'cBanana'):
+try:
+    from twisted.spread import cBanana
+except ImportError:
+    pass
+else:
+    # XXX: Eww.  It shouldn't be this hard to install cBanana.
+    banana.cBanana = cBanana
+    cBanana.pyb1282int = banana.b1282int
+    cBanana.pyint2b128 = banana.int2b128
     class CananaTestCase(BananaTestCase):
         encClass = banana.Canana
 
