@@ -1,18 +1,6 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """FTP tests.
 
@@ -25,6 +13,7 @@ from __future__ import  nested_scopes
 
 import sys, types, os.path, re
 from StringIO import StringIO
+from zope.interface import implements
 
 from twisted import internet
 from twisted.trial import unittest
@@ -254,7 +243,7 @@ bogusDirs = {
 }
 
 class BogusAvatar(object):
-    __implements__ = (ftp.IFTPShell,)
+    implements(ftp.IFTPShell)
 
     filesize = None
     
@@ -423,6 +412,7 @@ class TestFTPServer(FTPTestCase):
         iop.flush()
         reply = cli.lines[-1]
         self.assert_(re.search(r'227 =.*,[0-2]?[0-9]?[0-9],[0-2]?[0-9]?[0-9]',cli.lines[-1]))
+        self.cnx.s.cleanupDTP()
 
     def testTYPE(self):
         cli, sr, iop, send = self.cnx.getCSTuple()
@@ -779,7 +769,7 @@ class FTPClientTests(unittest.TestCase):
             f.noisy = 0
             f.protocol = protocol.Protocol
             port = reactor.listenTCP(0, f, interface="127.0.0.1")
-            n = port.getHost()[2]
+            n = port.getHost().port
             # This test data derived from a bug report by ranty on #twisted
             responses = ['220 ready, dude (vsFTPd 1.0.0: beat me, break me)',
                          # USER anonymous
