@@ -1,19 +1,7 @@
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """Tests for Perspective Broker module.
 
@@ -24,8 +12,10 @@ only specific tests for old API.
 import sys, os
 
 from cStringIO import StringIO
+from zope.interface import implements
 
 from twisted.trial import unittest
+from twisted.trial.util import suppressWarnings
 dR = unittest.deferredResult
 
 from twisted.spread import pb, util
@@ -126,7 +116,12 @@ def connectedServerAndClient():
     # Challenge-response authentication:
     pump.flush()
     return c, s, pump
-
+connectedServerAndClient = suppressWarnings(connectedServerAndClient,
+                                            ("twisted.internet.app", DeprecationWarning),
+                                            ("Identities", DeprecationWarning),
+                                            ("Cred services", DeprecationWarning),
+                                            ("This is deprecated. Use PBServerFactory.", DeprecationWarning),
+                                            ("Authorizers are deprecated, switch to portals/realms/etc.", DeprecationWarning))
 class SimpleRemote(pb.Referenceable):
     def remote_thunk(self, arg):
         self.arg = arg
@@ -542,6 +537,9 @@ class BrokerTestCase(unittest.TestCase):
         accum.pop().callRemote('doNothing').addCallback(accum.append)
         pump.flush()
         assert accum.pop() == 'hello world!', 'oops...'
+    testViewPoint = suppressWarnings(testViewPoint, 
+                                     ("This is deprecated. Use PBClientFactory.", DeprecationWarning),
+                                     ("pb.Perspective is deprecated, please use pb.Avatar.", DeprecationWarning))
 
     def testPublishable(self):
         import os
@@ -836,7 +834,12 @@ class ConnectionTestCase(unittest.TestCase):
         auth.addIdentity(ident2)
         self.svr = pb.BrokerFactory(pb.AuthRoot(auth))
         self.port = reactor.listenTCP(0, self.svr, interface="127.0.0.1")
-        self.portno = self.port.getHost()[-1]
+        self.portno = self.port.getHost().port
+    setUp = suppressWarnings(setUp,
+                             ("Update your backend to use PBServerFactory, and then use login().", DeprecationWarning),
+                             ("This is deprecated. Use PBServerFactory.", DeprecationWarning),
+                             ("Identities are deprecated, switch to credentialcheckers etc.", DeprecationWarning),
+                             ("Cred services are deprecated, use realms instead.", DeprecationWarning))
 
     def tearDown(self):
         self.port.stopListening()
@@ -852,12 +855,17 @@ class ConnectionTestCase(unittest.TestCase):
         root = dR(pb.getObjectAt("127.0.0.1", self.portno))
         self._checkRootObject(root)
         root.broker.transport.loseConnection()
-                       
+    testGetObjectAt = suppressWarnings(testGetObjectAt,
+                                       ("This is deprecated. Use PBClientFactory.", DeprecationWarning)) 
+                          
     def testConnect(self):
         p = dR(pb.connect("127.0.0.1", self.portno, "guest", "guest", "test",
                           perspectiveName="any"))
         self.assert_(isinstance(p, pb.RemoteReference))
         p.broker.transport.loseConnection()
+    testConnect = suppressWarnings(testConnect, 
+                                   ("Update your backend to use PBServerFactory, and then use login().", DeprecationWarning),
+                                   ("This is deprecated. Use PBClientFactory.", DeprecationWarning))
 
     def testIdentityConnector(self):
         iConnector = pb.IdentityConnector("127.0.0.1", self.portno, "guest", "guest")
@@ -866,6 +874,8 @@ class ConnectionTestCase(unittest.TestCase):
         self.assert_(isinstance(p1, pb.RemoteReference))
         self.assert_(isinstance(p2, pb.RemoteReference))
         iConnector.disconnect()
+    testIdentityConnector = suppressWarnings(testIdentityConnector, 
+                                            ("This is deprecated. Use PBClientFactory.", DeprecationWarning))
 
     # tests for new, shiny API, although getPerspective stuff is also deprecated:
     def testGoodGetObject(self):
@@ -890,6 +900,8 @@ class ConnectionTestCase(unittest.TestCase):
         p = dR(d)
         self.assert_(isinstance(p, pb.RemoteReference))
         p.broker.transport.loseConnection()
+    testGoodPerspective = suppressWarnings(testGoodPerspective, 
+                                           ("Update your backend to use PBServerFactory, and then use login().", DeprecationWarning))
     
     def testGoodFailedConnect(self):
         factory = pb.PBClientFactory()
@@ -898,6 +910,8 @@ class ConnectionTestCase(unittest.TestCase):
         f = unittest.deferredError(d)
         from twisted.internet import error
         f.trap(error.ConnectError)
+    testGoodFailedConnect = suppressWarnings(testGoodFailedConnect, 
+                                             ("Update your backend to use PBServerFactory, and then use login().", DeprecationWarning))
 
     def testDisconnect(self):
         factory = pb.PBClientFactory()
@@ -908,6 +922,8 @@ class ConnectionTestCase(unittest.TestCase):
         factory.disconnect()
         reactor.iterate(); reactor.iterate(); reactor.iterate()
         self.assertRaises(pb.DeadReferenceError, p.callRemote, "getDummyViewPoint")
+    testDisconnect = suppressWarnings(testDisconnect, 
+                                      ("Update your backend to use PBServerFactory, and then use login().", DeprecationWarning))
 
     def testEmptyPerspective(self):
         factory = pb.PBClientFactory()
@@ -916,6 +932,8 @@ class ConnectionTestCase(unittest.TestCase):
         p = dR(d)
         self.assert_(isinstance(p, pb.RemoteReference))
         p.broker.transport.loseConnection()
+    testEmptyPerspective = suppressWarnings(testEmptyPerspective, 
+                                            ("Update your backend to use PBServerFactory, and then use login().", DeprecationWarning))
 
     def testReconnect(self):
         factory = ReconnectOnce()
@@ -934,6 +952,8 @@ class ConnectionTestCase(unittest.TestCase):
         p = dR(l[0])
         self.assert_(isinstance(p, pb.RemoteReference))
         factory.disconnect()
+    testReconnect = suppressWarnings(testReconnect,
+                                     ("Update your backend to use PBServerFactory, and then use login().", DeprecationWarning))
 
     def testImmediateClose(self):
         from twisted.internet import protocol
@@ -961,7 +981,7 @@ class MyRealm:
 
 class MyPerspective(pb.Avatar):
 
-    __implements__ = pb.IPerspective,
+    implements(pb.IPerspective)
 
     loggedIn = loggedOut = False
 
@@ -990,7 +1010,7 @@ class NewCredTestCase(unittest.TestCase):
         self.portal.registerChecker(self.checker)
         self.factory = pb.PBServerFactory(self.portal)
         self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
-        self.portno = self.port.getHost()[-1]
+        self.portno = self.port.getHost().port
 
     def tearDown(self):
         self.port.stopListening()
@@ -1016,7 +1036,7 @@ class NewCredTestCase(unittest.TestCase):
     def testBadLogin(self):
         factory = pb.PBClientFactory()
         for username, password in [("nosuchuser", "pass"), ("user", "wrongpass")]:
-            d = factory.login(credentials.UsernamePassword("user1", "pass"), "BRAINS!")
+            d = factory.login(credentials.UsernamePassword(username, password), "BRAINS!")
             c = reactor.connectTCP("127.0.0.1", self.portno, factory)
             p = unittest.deferredError(d)
             self.failUnless(p.check("twisted.cred.error.UnauthorizedLogin"))
@@ -1037,7 +1057,7 @@ class NewCredTestCase(unittest.TestCase):
         factory.disconnect()
 
 class NonSubclassingPerspective:
-    __implements__ = pb.IPerspective,
+    implements(pb.IPerspective)
 
     # IPerspective implementation
     def perspectiveMessageReceived(self, broker, message, args, kwargs):
@@ -1059,14 +1079,13 @@ class NSPTestCase(unittest.TestCase):
         self.portal.registerChecker(self.checker)
         self.factory = pb.PBServerFactory(self.portal)
         self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
-        self.portno = self.port.getHost()[-1]
+        self.portno = self.port.getHost().port
 
     def tearDown(self):
         self.port.stopListening()
         reactor.iterate()
         reactor.iterate()
 
-
     def testNSP(self):
         factory = pb.PBClientFactory()
         d = factory.login(credentials.UsernamePassword('user', 'pass'), "BRAINS!")
@@ -1075,4 +1094,3 @@ class NSPTestCase(unittest.TestCase):
         self.assertEquals(dR(p.callRemote('ANYTHING', 'here', bar='baz')),
                           ('ANYTHING', ('here',), {'bar': 'baz'}))
         factory.disconnect()
-
