@@ -1,26 +1,16 @@
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """
 Test running processes.
 """
-from __future__ import nested_scopes
+from __future__ import nested_scopes, generators
 
 from twisted.trial import unittest
+from twisted.trial.util import spinUntil, spinWhile
+from twisted.python import log
 
 import gzip
 import os
@@ -28,6 +18,7 @@ import popen2
 import time
 import sys
 import signal
+import shutil
 
 try:
     import cStringIO as StringIO
@@ -35,9 +26,9 @@ except ImportError:
     import StringIO
 
 # Twisted Imports
-from twisted.internet import reactor, protocol, error, interfaces
+from twisted.internet import reactor, protocol, error, interfaces, defer
 from twisted.python import util, runtime, components
-from twisted.runner import procutils
+from twisted.python import procutils
 
 class TrivialProcessProtocol(protocol.ProcessProtocol):
     finished = 0
@@ -100,9 +91,10 @@ class EchoProtocol(protocol.ProcessProtocol):
 
     def processEnded(self, reason):
         self.finished = 1
-        if not isinstance(reason.value, error.ProcessDone):
-            print reason
-            raise "process didn't terminate normally"
+        if not reason.check(error.ProcessDone):
+            self.failure = "process didn't terminate normally: %s" % reason
+            return
+        self.failure = None
 
 class SignalProtocol(protocol.ProcessProtocol):
     def __init__(self, sig, testcase):
@@ -115,54 +107,45 @@ class SignalProtocol(protocol.ProcessProtocol):
 
     def processEnded(self, reason):
         self.going = 0
-        reason.trap(error.ProcessTerminated)
+        if not reason.check(error.ProcessTerminated):
+            self.failure = "wrong termination: %s" % reason
+            return
         v = reason.value
-        self.testcase.assertEquals(v.exitCode, None,
-                                   "SIG%s: exitCode is %s, not None" % \
-                                   (self.signal, v.exitCode))
-        self.testcase.assertEquals(v.signal,
-                                   getattr(signal,'SIG'+self.signal),
-                                   "SIG%s: .signal was %s, wanted %s" % \
-                                   (self.signal, v.signal,
-                                    getattr(signal,'SIG'+self.signal)))
-        self.testcase.assertEquals(os.WTERMSIG(v.status),
-                                   getattr(signal,'SIG'+self.signal),
-                                   'SIG%s: %s' % (self.signal,
-                                                  os.WTERMSIG(v.status)))
+        if v.exitCode is not None:
+            self.failure = "SIG%s: exitCode is %s, not None" % \
+                           (self.signal, v.exitCode)
+            return
+        if v.signal != getattr(signal,'SIG'+self.signal):
+            self.failure = "SIG%s: .signal was %s, wanted %s" % \
+                           (self.signal, v.signal,
+                            getattr(signal,'SIG'+self.signal))
+            return
+        if os.WTERMSIG(v.status) != getattr(signal,'SIG'+self.signal):
+            self.failure = 'SIG%s: %s' % (self.signal,
+                                          os.WTERMSIG(v.status))
+            return
+        self.failure = None
 
 class SignalMixin:
     sigchldHandler = None
     
     def setUpClass(self):
-        # make sure SIGCHLD handler is installed, as it should be on reactor.run().
-        # problem is reactor may not have been run when this test runs.
+        # make sure SIGCHLD handler is installed, as it should be on
+        # reactor.run(). Do this because the reactor may not have been run
+        # by the time this test runs.
         if hasattr(reactor, "_handleSigchld") and hasattr(signal, "SIGCHLD"):
-            self.sigchldHandler = signal.signal(signal.SIGCHLD, reactor._handleSigchld)
+            log.msg("Installing SIGCHLD signal handler.")
+            self.sigchldHandler = signal.signal(signal.SIGCHLD,
+                                                reactor._handleSigchld)
+        else:
+            log.msg("Skipped installing SIGCHLD signal handler.")
     
     def tearDownClass(self):
         if self.sigchldHandler:
+            log.msg("Uninstalled SIGCHLD signal handler.")
             signal.signal(signal.SIGCHLD, self.sigchldHandler)
 
 
-class PausingProcessProtocol(protocol.ProcessProtocol):
-
-    data = ""
-    elapsed = None
-    
-    def connectionMade(self):
-        self.transport.pauseProducing()
-        self.transport.write("a")
-        reactor.callLater(2, self.transport.resumeProducing)
-    
-    def outReceived(self, d):
-        self.data += d
-
-    def processEnded(self, reason):
-        self.data = self.data.lstrip("a")
-        if len(self.data) != 5: raise ValueError
-        self.elapsed = float(self.data)
-
-
 class ProcessTestCase(SignalMixin, unittest.TestCase):
     """Test running a process."""
 
@@ -172,9 +155,7 @@ class ProcessTestCase(SignalMixin, unittest.TestCase):
         p = TestProcessProtocol()
         reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None)
 
-        timeout = time.time() + 10
-        while not p.finished and not (time.time() > timeout):
-            reactor.iterate(0.01)
+        spinUntil(lambda :p.finished, 10)
         self.failUnless(p.finished)
         self.assertEquals(p.stages, [1, 2, 3, 4, 5])
 
@@ -196,20 +177,11 @@ class ProcessTestCase(SignalMixin, unittest.TestCase):
         scriptPath = util.sibpath(__file__, "process_echoer.py")
         p = EchoProtocol()
         reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None)
-        while not p.finished:
-            reactor.iterate(0.01)
+        spinUntil(lambda :p.finished, 20)
+        self.failIf(p.failure, p.failure)
         self.assert_(hasattr(p, 'buffer'))
         self.assertEquals(len(p.buffer), len(p.s * 10))
 
-    def testPausing(self):
-        exe = sys.executable
-        scriptPath = util.sibpath(__file__, "process_pausing.py")
-        p = PausingProcessProtocol()
-        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None)
-        while p.elapsed == None:
-            reactor.iterate(0.01)
-        self.assert_(2.1 > p.elapsed > 1.5) # assert how long process was paused
-
 
 class TwoProcessProtocol(protocol.ProcessProtocol):
     finished = 0
@@ -224,13 +196,7 @@ class TestTwoProcessesBase:
         self.processes = [None, None]
         self.pp = [None, None]
         self.done = 0
-        self.timeout = None
         self.verbose = 0
-    def tearDown(self):
-        if self.timeout:
-            self.timeout.cancel()
-            self.timeout = None
-        # I don't think I can use os.kill outside of POSIX, so skip cleanup
 
     def createProcesses(self, usePTY=0):
         exe = sys.executable
@@ -250,36 +216,26 @@ class TestTwoProcessesBase:
         self.failIf(pp.finished, "Process finished too early")
         p.loseConnection()
         if self.verbose: print self.pp[0].finished, self.pp[1].finished
-
-    def giveUp(self):
-        self.timeout = None
-        self.done = 1
-        if self.verbose: print "timeout"
-        self.fail("timeout")
         
     def check(self):
         #print self.pp[0].finished, self.pp[1].finished
         #print "  ", self.pp[0].num, self.pp[1].num
         if self.pp[0].finished and self.pp[1].finished:
             self.done = 1
+        return self.done
             
     def testClose(self):
         if self.verbose: print "starting processes"
         self.createProcesses()
         reactor.callLater(1, self.close, 0)
         reactor.callLater(2, self.close, 1)
-        self.timeout = reactor.callLater(5, self.giveUp)
-        self.check()
-        while not self.done:
-            reactor.iterate(0.01)
-            self.check()
+        spinUntil(self.check, 5)
 
 class TestTwoProcessesNonPosix(TestTwoProcessesBase, SignalMixin, unittest.TestCase):
     pass
 
 class TestTwoProcessesPosix(TestTwoProcessesBase, SignalMixin, unittest.TestCase):
     def tearDown(self):
-        TestTwoProcessesBase.tearDown(self)
         self.check()
         for i in (0,1):
             pp, process = self.pp[i], self.processes[i]
@@ -288,13 +244,7 @@ class TestTwoProcessesPosix(TestTwoProcessesBase, SignalMixin, unittest.TestCase
                     os.kill(process.pid, signal.SIGTERM)
                 except OSError:
                     print "OSError"
-        now = time.time()
-        self.check()
-        while not self.done or (time.time() > now + 5):
-            reactor.iterate(0.01)
-            self.check()
-        if not self.done:
-            print "unable to shutdown child processes"
+        spinUntil(self.check, 5, msg="unable to shutdown child processes")
 
     def kill(self, num):
         if self.verbose: print "kill [%d] with SIGTERM" % num
@@ -309,33 +259,21 @@ class TestTwoProcessesPosix(TestTwoProcessesBase, SignalMixin, unittest.TestCase
         self.createProcesses(usePTY=0)
         reactor.callLater(1, self.kill, 0)
         reactor.callLater(2, self.kill, 1)
-        self.timeout = reactor.callLater(5, self.giveUp)
-        self.check()
-        while not self.done:
-            reactor.iterate(0.01)
-            self.check()
+        spinUntil(self.check, 5)
 
     def testClosePty(self):
         if self.verbose: print "starting processes"
         self.createProcesses(usePTY=1)
         reactor.callLater(1, self.close, 0)
         reactor.callLater(2, self.close, 1)
-        self.timeout = reactor.callLater(5, self.giveUp)
-        self.check()
-        while not self.done:
-            reactor.iterate(0.01)
-            self.check()
+        spinUntil(self.check, 5)
     
     def testKillPty(self):
         if self.verbose: print "starting processes"
         self.createProcesses(usePTY=1)
         reactor.callLater(1, self.kill, 0)
         reactor.callLater(2, self.kill, 1)
-        self.timeout = reactor.callLater(5, self.giveUp)
-        self.check()
-        while not self.done:
-            reactor.iterate(0.01)
-            self.check()
+        spinUntil(self.check, 5)
 
 class FDChecker(protocol.ProcessProtocol):
     state = 0
@@ -434,12 +372,12 @@ class FDTest(SignalMixin, unittest.TestCase):
                              path=None,
                              childFDs={0:"w", 1:"r", 2:2,
                                        3:"w", 4:"r", 5:"w"})
-        timeout = time.time() + 5
-        while not p.done and time.time() < timeout:
-            reactor.iterate(0.01)
-        self.failUnless(p.done, "timeout")
+        spinUntil(lambda :p.done, 5)
         self.failIf(p.failed, p.failed)
 
+    if sys.platform.find("freebsd") != -1:
+        testFD.todo = "This test fails on freebsd5 - slyphon"
+
     def testLinger(self):
         # See what happens when all the pipes close before the process
         # actually stops. This test *requires* SIGCHLD catching to work,
@@ -451,10 +389,7 @@ class FDTest(SignalMixin, unittest.TestCase):
                              path=None,
                              childFDs={1:"r", 2:2},
                              )
-        timeout = time.time() + 7
-        while not p.closed and time.time() < timeout:
-            reactor.iterate(0.01)
-        self.failUnless(p.closed, "timeout")
+        spinUntil(lambda :p.closed, 7)
         self.failUnlessEqual(p.outF.getvalue(),
                              "here is some text\ngoodbye\n")
 
@@ -501,8 +436,7 @@ class PosixProcessBase:
         reactor.spawnProcess(p, cmd, ['true'], env=None,
                              usePTY=self.usePTY)
 
-        while not p.finished:
-            reactor.iterate(0.01)
+        spinUntil(lambda :p.finished)
         p.reason.trap(error.ProcessDone)
         self.assertEquals(p.reason.value.exitCode, 0)
         self.assertEquals(p.reason.value.signal, None)
@@ -516,8 +450,7 @@ class PosixProcessBase:
         reactor.spawnProcess(p, cmd, ['false'], env=None,
                              usePTY=self.usePTY)
 
-        while not p.finished:
-            reactor.iterate(0.01)
+        spinUntil(lambda :p.finished)
         p.reason.trap(error.ProcessTerminated)
         self.assertEquals(p.reason.value.exitCode, 1)
         self.assertEquals(p.reason.value.signal, None)
@@ -526,16 +459,13 @@ class PosixProcessBase:
         exe = sys.executable
         scriptPath = util.sibpath(__file__, "process_signal.py")
         signals = ('HUP', 'INT', 'KILL')
-        protocols = []
         for sig in signals:
             p = SignalProtocol(sig, self)
             reactor.spawnProcess(p, exe, [exe, "-u", scriptPath, sig],
                                  env=None,
                                  usePTY=self.usePTY)
-            protocols.append(p)
-
-        while reduce(lambda a,b:a+b,[p.going for p in protocols]):
-            reactor.iterate(0.01)
+            spinWhile(lambda :p.going)
+            self.failIf(p.failure, p.failure)
 
 class PosixProcessTestCase(SignalMixin, unittest.TestCase, PosixProcessBase):
     # add three non-pty test cases
@@ -545,21 +475,33 @@ class PosixProcessTestCase(SignalMixin, unittest.TestCase, PosixProcessBase):
         exe = sys.executable
         scriptPath = util.sibpath(__file__, "process_twisted.py")
         p = Accumulator()
-        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None,
+        # As trial chdirs to _trial_temp after startup, this makes any
+        # possible relative entries in PYTHONPATH invalid. Attempt to
+        # fix that up.  Otherwise process_twisted.py will use the
+        # installed Twisted, which isn't really guaranteed to exist at
+        # this stage.
+        def fixup(l):
+            for path in l:
+                if os.path.isabs(path):
+                    yield path
+                else:
+                    yield os.path.join(os.path.pardir, path)
+        env = {"PYTHONPATH": os.pathsep.join(fixup(sys.path))}
+        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=env,
                              path=None, usePTY=self.usePTY)
         p.transport.write("hello, world")
         p.transport.write("abc")
         p.transport.write("123")
         p.transport.closeStdin()
-        timeout = time.time() + 10
-        while not p.closed and not (time.time() > timeout):
-            reactor.iterate(0.01)
-        self.failUnless(p.closed)
-        self.assertEquals(p.outF.getvalue(), "hello, worldabc123", "Error message from process_twisted follows:\n\n%s\n\n" % p.errF.getvalue())
+        spinUntil(lambda :p.closed, 10)
+        self.assertEquals(p.outF.getvalue(), "hello, worldabc123",
+                          "Error message from process_twisted follows:"
+                          "\n\n%s\n\n" % p.errF.getvalue())
 
     def testStderr(self):
         # we assume there is no file named ZZXXX..., both in . and in /tmp
-        if not os.path.exists('/bin/ls'): raise RuntimeError("/bin/ls not found")
+        if not os.path.exists('/bin/ls'):
+            raise RuntimeError("/bin/ls not found")
 
         p = Accumulator()
         reactor.spawnProcess(p, '/bin/ls',
@@ -568,8 +510,7 @@ class PosixProcessTestCase(SignalMixin, unittest.TestCase, PosixProcessBase):
                              env=None, path="/tmp",
                              usePTY=self.usePTY)
 
-        while not p.closed:
-            reactor.iterate(0.01)
+        spinUntil(lambda :p.closed)
         self.assertEquals(lsOut, p.errF.getvalue())
 
     def testProcess(self):
@@ -583,10 +524,7 @@ class PosixProcessTestCase(SignalMixin, unittest.TestCase, PosixProcessBase):
         p.transport.write(s)
         p.transport.closeStdin()
 
-        timeout = time.time() + 10
-        while not p.closed and not (time.time() > timeout):
-            reactor.iterate(0.01)
-        self.failUnless(p.closed)
+        spinUntil(lambda :p.closed, 10)
         f = p.outF
         f.seek(0, 0)
         gf = gzip.GzipFile(fileobj=f)
@@ -601,6 +539,17 @@ class PosixProcessTestCasePTY(SignalMixin, unittest.TestCase, PosixProcessBase):
     # testSignal
     # testProcess, but not without p.transport.closeStdin
     #  might be solveable: TODO: add test if so
+
+    def testOpeningTTY(self):
+        exe = sys.executable
+        scriptPath = util.sibpath(__file__, "process_tty.py")
+        p = Accumulator()
+        
+        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None,
+                            path=None, usePTY=self.usePTY)
+        p.transport.write("hello world!\n")
+        spinUntil(lambda :p.closed, 10)
+        self.assertEquals(p.outF.getvalue(), "hello world!\r\nhello world!\r\n", "Error message from process_tty follows:\n\n%s\n\n" % p.outF.getvalue())
     
 class Win32ProcessTestCase(SignalMixin, unittest.TestCase):
     """Test process programs that are packaged with twisted."""
@@ -614,8 +563,7 @@ class Win32ProcessTestCase(SignalMixin, unittest.TestCase):
         p.transport.write("hello, world")
         p.transport.closeStdin()
 
-        while not p.closed:
-            reactor.iterate(0.01)
+        spinUntil(lambda :p.closed)
         self.assertEquals(p.errF.getvalue(), "err\nerr\n")
         self.assertEquals(p.outF.getvalue(), "out\nhello, world\nout\n")
 
@@ -628,6 +576,8 @@ class UtilTestCase(unittest.TestCase):
         barfoo = j("baz", "bar")
         
         for d in "foo", foobar, foobaz, "baz", bazfoo, barfoo:
+            if os.path.exists(d):
+                shutil.rmtree(d, True)
             os.mkdir(d)
         
         f = file(j(foobaz, "executable"), "w")
@@ -648,13 +598,28 @@ class UtilTestCase(unittest.TestCase):
         
         f = file(j(barfoo, "executable"), "w")
         f.close()
-        
+      
         klass.oldPath = os.environ['PATH']
         os.environ['PATH'] = os.pathsep.join((foobar, foobaz, bazfoo, barfoo))
     
     def tearDownClass(klass):
+        j = os.path.join
         os.environ['PATH'] = klass.oldPath
-    
+        foobar = j("foo", "bar")
+        foobaz = j("foo", "baz")
+        bazfoo = j("baz", "foo")
+        barfoo = j("baz", "bar")
+        
+       
+        os.remove(j(foobaz, "executable"))
+        os.remove(j("foo", "executable"))
+        os.remove(j(bazfoo, "executable"))
+        os.remove(j(bazfoo, "executable.bin"))
+        os.remove(j(barfoo, "executable"))
+
+        for d in foobar, foobaz, bazfoo, barfoo, "foo", "baz":
+            os.rmdir(d)
+     
     def testWhich(self):
         j = os.path.join
         paths = procutils.which("executable")
@@ -678,8 +643,77 @@ class UtilTestCase(unittest.TestCase):
             j("baz", "foo", "executable.bin")
         ])
 
+class ClosingPipesProcessProtocol(protocol.ProcessProtocol):
+    output = ''
+    errput = ''
+
+    def __init__(self, outOrErr):
+        self.deferred = defer.Deferred()
+        self.deferred.addCallbacks(
+            callback=lambda _: unittest.fail("I wanted an errback."),
+            errback=self._end)
+        self.outOrErr = outOrErr
+
+    def _end(self, reason):
+        unittest.failIf(reason.check(error.ProcessDone),
+                        'Child should fail due to EPIPE.')
+        reason.trap(error.ProcessTerminated)
+        # child must not get past that write without raising
+        unittest.failIfEqual(reason.value.exitCode, 42)
+        unittest.failUnlessEqual(self.output, '')
+        return self.errput
+
+    def processEnded(self, reason):
+        self.deferred.callback(reason)
+
+    def outReceived(self, data):
+        self.output += data
+
+    def errReceived(self, data):
+        self.errput += data
+
+class ClosingPipes(unittest.TestCase):
+
+    def doit(self, fd):
+        p = ClosingPipesProcessProtocol(True)
+        reactor.spawnProcess(p, sys.executable,
+                             [sys.executable, '-c', r'raw_input(); import sys, os; os.write(%d, "foo\n"); sys.exit(42)' % fd],
+                             env=None)
+        p.transport.write('go\n')
+
+        if fd == 1:
+            p.transport.closeStdout()
+        elif fd == 2:
+            p.transport.closeStderr()
+        else:
+            raise RuntimeError
+
+        # make the buggy case not hang
+        p.transport.closeStdin()
+        return p.deferred
+
+    def test_stdout(self):
+        """ProcessProtocol.transport.closeStdout actually closes the pipe."""
+        d = self.doit(1)
+        def _check(errput):
+            unittest.failIfEqual(errput.index('OSError'), -1)
+            unittest.failIfEqual(errput.index('Broken pipe'), -1)
+        d.addCallback(_check)
+        return d
+
+    def test_stderr(self):
+        """ProcessProtocol.transport.closeStderr actually closes the pipe."""
+        d = self.doit(2)
+        def _check(errput):
+            # there should be no stderr open, so nothing for it to
+            # write the error to.
+            unittest.failUnlessEqual(errput, '')
+        d.addCallback(_check)
+        return d
+
+
 skipMessage = "wrong platform or reactor doesn't support IReactorProcess"
-if (runtime.platform.getType() != 'posix') or (not components.implements(reactor, interfaces.IReactorProcess)):
+if (runtime.platform.getType() != 'posix') or (not interfaces.IReactorProcess(reactor, None)):
     PosixProcessTestCase.skip = skipMessage
     PosixProcessTestCasePTY.skip = skipMessage
     TestTwoProcessesPosix.skip = skipMessage
@@ -688,7 +722,7 @@ else:
     # do this before running the tests: it uses SIGCHLD and stuff internally
     lsOut = popen2.popen3("/bin/ls ZZXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")[2].read()
 
-if (runtime.platform.getType() != 'win32') or (not components.implements(reactor, interfaces.IReactorProcess)):
+if (runtime.platform.getType() != 'win32') or (not interfaces.IReactorProcess(reactor, None)):
     Win32ProcessTestCase.skip = skipMessage
     TestTwoProcessesNonPosix.skip = skipMessage
 
@@ -696,5 +730,6 @@ if runtime.platform.getType() == 'win32':
     ProcessTestCase.testEcho.im_func.todo = "goes into infinite loop in win32eventreactor :("
     UtilTestCase.todo = "do not assume that platform retains 'executable' mode"
 
-if (not components.implements(reactor, interfaces.IReactorProcess)):
+if not interfaces.IReactorProcess(reactor, None):
     ProcessTestCase.skip = skipMessage
+    ClosingPipes.skip = skipMessage
