@@ -1,26 +1,14 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """
 Test cases for twisted.protocols package.
 """
 
 from twisted.trial import unittest
-from twisted.protocols import basic, wire
-from twisted.internet import reactor, protocol
+from twisted.protocols import basic, wire, portforward
+from twisted.internet import reactor, protocol, defer
 
 import string, struct
 import StringIO
@@ -41,7 +29,17 @@ class LineTester(basic.LineReceiver):
         self.received.append(line)
         if line == '':
             self.setRawMode()
-        if line[:4] == 'len ':
+        elif line == 'pause':
+            self.pauseProducing()
+            reactor.callLater(0, self.resumeProducing)
+        elif line == 'rawpause':
+            self.pauseProducing()
+            self.setRawMode()
+            self.received.append('')
+            reactor.callLater(0, self.resumeProducing)
+        elif line == 'stop':
+            self.stopProducing()
+        elif line[:4] == 'len ':
             self.length = int(line[4:])
 
     def rawDataReceived(self, data):
@@ -138,6 +136,55 @@ a'''
                 a.dataReceived(s)
             self.failUnlessEqual(self.output, a.received)
 
+
+    pause_buf = 'twiddle1\ntwiddle2\npause\ntwiddle3\n'
+
+    pause_output1 = ['twiddle1', 'twiddle2', 'pause']
+    pause_output2 = pause_output1+['twiddle3']
+
+    def testPausing(self):
+        for packet_size in range(1, 10):
+            t = StringIOWithoutClosing()
+            a = LineTester()
+            a.makeConnection(protocol.FileWrapper(t))
+            for i in range(len(self.pause_buf)/packet_size + 1):
+                s = self.pause_buf[i*packet_size:(i+1)*packet_size]
+                a.dataReceived(s)
+            self.failUnlessEqual(self.pause_output1, a.received)
+            reactor.iterate(0)
+            self.failUnlessEqual(self.pause_output2, a.received)
+
+    rawpause_buf = 'twiddle1\ntwiddle2\nlen 5\nrawpause\n12345twiddle3\n'
+
+    rawpause_output1 = ['twiddle1', 'twiddle2', 'len 5', 'rawpause', '']
+    rawpause_output2 = ['twiddle1', 'twiddle2', 'len 5', 'rawpause', '12345', 'twiddle3']
+
+    def testRawPausing(self):
+        for packet_size in range(1, 10):
+            t = StringIOWithoutClosing()
+            a = LineTester()
+            a.makeConnection(protocol.FileWrapper(t))
+            for i in range(len(self.rawpause_buf)/packet_size + 1):
+                s = self.rawpause_buf[i*packet_size:(i+1)*packet_size]
+                a.dataReceived(s)
+            self.failUnlessEqual(self.rawpause_output1, a.received)
+            reactor.iterate(0)
+            self.failUnlessEqual(self.rawpause_output2, a.received)
+
+    stop_buf = 'twiddle1\ntwiddle2\nstop\nmore\nstuff\n'
+
+    stop_output = ['twiddle1', 'twiddle2', 'stop']
+    def testStopProducing(self):
+        for packet_size in range(1, 10):
+            t = StringIOWithoutClosing()
+            a = LineTester()
+            a.makeConnection(protocol.FileWrapper(t))
+            for i in range(len(self.stop_buf)/packet_size + 1):
+                s = self.stop_buf[i*packet_size:(i+1)*packet_size]
+                a.dataReceived(s)
+            self.failUnlessEqual(self.stop_output, a.received)
+        
+
 class LineOnlyReceiverTestCase(unittest.TestCase):
 
     buffer = """foo
@@ -250,3 +297,40 @@ class Int32TestCase(unittest.TestCase, LPTestCaseMixin):
             for c in struct.pack("!i",len(s))+s:
                 r.dataReceived(c)
         self.assertEquals(r.received, self.strings)
+
+class Portforwarding(unittest.TestCase):
+    def testPortforward(self):
+        serverProtocol = wire.Echo()
+        realServerFactory = protocol.ServerFactory()
+        realServerFactory.protocol = lambda: serverProtocol
+        realServerPort = reactor.listenTCP(0, realServerFactory,
+                                           interface='127.0.0.1')
+
+        proxyServerFactory = portforward.ProxyFactory('127.0.0.1',
+                                                      realServerPort.getHost().port)
+        proxyServerPort = reactor.listenTCP(0, proxyServerFactory,
+                                            interface='127.0.0.1')
+
+        nBytes = 1000
+        received = []
+        clientProtocol = protocol.Protocol()
+        clientProtocol.dataReceived = received.extend
+        clientProtocol.connectionMade = lambda: clientProtocol.transport.write('x' * nBytes)
+        clientFactory = protocol.ClientFactory()
+        clientFactory.protocol = lambda: clientProtocol
+
+        reactor.connectTCP('127.0.0.1', proxyServerPort.getHost().port,
+                           clientFactory)
+
+        c = 0
+        while len(received) < nBytes and c < 100:
+            reactor.iterate(0.01)
+            c += 1
+
+        self.assertEquals(''.join(received), 'x' * nBytes)
+        
+        clientProtocol.transport.loseConnection()
+        serverProtocol.transport.loseConnection()
+        return defer.gatherResults([
+            defer.maybeDeferred(realServerPort.stopListening),
+            defer.maybeDeferred(proxyServerPort.stopListening)])
