@@ -1,20 +1,9 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001-2002 Matthew W. Lefkowitz
+# -*- test-case-name: twisted.test.test_udp -*-
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 # 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-# 
-from twisted.trial import unittest
+from twisted.trial import unittest, util
 
 from twisted.internet import protocol, reactor, error, defer, interfaces, address
 from twisted.python import failure, components
@@ -59,8 +48,35 @@ class GoodClient(Server):
         self.refused = 1
 
 
-class OldConnectedUDPTestCase(unittest.TestCase):
-        
+class PortCleanerUpper(unittest.TestCase):
+    callToLoseCnx = 'loseConnection'
+    def setUp(self):
+        self.ports = []
+
+    def tearDown(self):
+        self.cleanPorts(*self.ports)
+
+    def _addPorts(self, *ports):
+        for p in ports:
+            self.ports.append(p)
+
+    def cleanPorts(self, *ports):
+        for p in ports:
+            if not hasattr(p, 'disconnected'):
+                raise RuntimeError, ("You handed something to cleanPorts that"
+                                     " doesn't have a disconnected attribute, dummy!")
+            if not p.disconnected:
+                d = getattr(p, self.callToLoseCnx)()
+                if isinstance(d, defer.Deferred):
+                    wait(d)
+                else:
+                    try:
+                        util.spinUntil(lambda :p.disconnected)
+                    except:
+                        failure.Failure().printTraceback()
+
+
+class OldConnectedUDPTestCase(PortCleanerUpper):
     def testStartStop(self):
         client = Client()
         port2 = reactor.connectUDP("127.0.0.1", 8888, client)
@@ -69,11 +85,16 @@ class OldConnectedUDPTestCase(unittest.TestCase):
         self.assertEquals(client.started, 1)
         self.assertEquals(client.stopped, 0)
         l = []
+        self.assert_(repr(port2).find('test_udp.Client') > 0)
         defer.maybeDeferred(port2.stopListening).addCallback(l.append)
         reactor.iterate()
         reactor.iterate()
+        self.assert_(repr(port2).find('test_udp.Client') > 0)
         self.assertEquals(client.stopped, 1)
         self.assertEquals(len(l), 1)
+    testStartStop = util.suppressWarnings(testStartStop,
+                                          ('use listenUDP and then transport.connect',
+                                           DeprecationWarning))
 
     def testDNSFailure(self):
         client = Client()
@@ -84,24 +105,30 @@ class OldConnectedUDPTestCase(unittest.TestCase):
         self.assert_(client.failure.trap(error.DNSLookupError))
         self.assertEquals(client.stopped, 0)
         self.assertEquals(client.started, 0)
-
+    testDNSFailure = util.suppressWarnings(testDNSFailure,
+                                           ('use listenUDP and then transport.connect',
+                                           DeprecationWarning))
     def testSendPackets(self):
         server = Server()
         port1 = reactor.listenUDP(0, server, interface="127.0.0.1")
         client = Client()
-        port2 = reactor.connectUDP("127.0.0.1", server.transport.getHost()[2], client)
+        port2 = reactor.connectUDP("127.0.0.1", server.transport.getHost().port, client)
         reactor.iterate()
         reactor.iterate()
         reactor.iterate()
-        server.transport.write("hello", client.transport.getHost()[1:])
+        server.transport.write("hello", (client.transport.getHost().host,
+                                         client.transport.getHost().port))
         client.transport.write("world")
         reactor.iterate()
         reactor.iterate()
         reactor.iterate()
         self.assertEquals(client.packets, ["hello"])
-        self.assertEquals(server.packets, [("world", ("127.0.0.1", client.transport.getHost()[2]))])
+        self.assertEquals(server.packets, [("world", ("127.0.0.1", client.transport.getHost().port))])
         port1.stopListening(); port2.stopListening()
         reactor.iterate(); reactor.iterate()
+    testSendPackets = util.suppressWarnings(testSendPackets,
+                                            ('use listenUDP and then transport.connect',
+                                             DeprecationWarning))
 
     def testConnectionRefused(self):
         # assume no one listening on port 80 UDP
@@ -117,13 +144,18 @@ class OldConnectedUDPTestCase(unittest.TestCase):
         server.transport.write("c", ("127.0.0.1", 80))
         server.transport.write("d", ("127.0.0.1", 80))
         server.transport.write("e", ("127.0.0.1", 80))
-        server.transport.write("toserver", port2.getHost()[1:])
-        server.transport.write("toclient", port.getHost()[1:])
+        server.transport.write("toserver", (port2.getHost().host,
+                                            port2.getHost().port))
+        server.transport.write("toclient", (port.getHost().host,
+                                            port.getHost().port))
         reactor.iterate(); reactor.iterate()
         self.assertEquals(client.refused, 1)
         port.stopListening()
         port2.stopListening()
         reactor.iterate(); reactor.iterate()
+    testConnectionRefused = util.suppressWarnings(testConnectionRefused,
+                                                  ('use listenUDP and then transport.connect',
+                                                   DeprecationWarning))
 
 
 class UDPTestCase(unittest.TestCase):
@@ -135,6 +167,9 @@ class UDPTestCase(unittest.TestCase):
         addr = p.getHost()
         self.assertEquals(addr, ('INET_UDP', addr.host, addr.port))
         p.stopListening()
+    testOldAddress = util.suppressWarnings(testOldAddress,
+                                           ('IPv4Address.__getitem__',
+                                            DeprecationWarning))
     
     def testStartStop(self):
         server = Server()
@@ -164,7 +199,7 @@ class UDPTestCase(unittest.TestCase):
     def testBindError(self):
         server = Server()
         port = reactor.listenUDP(0, server, interface='127.0.0.1')
-        n = port.getHost()[2]
+        n = port.getHost().port
         reactor.iterate()
         reactor.iterate()
         reactor.iterate()
@@ -204,15 +239,13 @@ class UDPTestCase(unittest.TestCase):
         client.transport.connect("127.0.0.1", 80)
         server = Server()
         port2 = reactor.listenUDP(0, server, interface="127.0.0.1")
-        reactor.iterate()
-        reactor.iterate()
-        reactor.iterate()
+        self.runReactor(0.4, True)
         client.transport.write("a")
         client.transport.write("b")
         server.transport.write("c", ("127.0.0.1", 80))
         server.transport.write("d", ("127.0.0.1", 80))
         server.transport.write("e", ("127.0.0.1", 80))
-        reactor.iterate(); reactor.iterate()
+        self.runReactor(0.4, True)
         self.assertEquals(client.refused, 1)
         port.stopListening()
         port2.stopListening()
@@ -248,9 +281,10 @@ class MulticastTestCase(unittest.TestCase):
         self.client = Client()
         # multicast won't work if we listen over loopback, apparently
         self.port1 = reactor.listenMulticast(0, self.server)
-        self.port2 = reactor.connectMulticast("127.0.0.1", self.server.transport.getHost()[2], self.client)
+        self.port2 = reactor.listenMulticast(0, self.client)
         reactor.iterate()
         reactor.iterate()
+        self.client.transport.connect("127.0.0.1", self.server.transport.getHost().port)
 
     def tearDown(self):
         self.port1.stopListening()
@@ -269,17 +303,23 @@ class MulticastTestCase(unittest.TestCase):
             self.assertEquals(o.transport.getTTL(), 2)
 
     def testLoopback(self):
-        for o in self.client, self.server:
-            self.assertEquals(o.transport.getLoopbackMode(), 1)
-            o.transport.setLoopbackMode(0)
-            self.assertEquals(o.transport.getLoopbackMode(), 0)
+        self.assertEquals(self.server.transport.getLoopbackMode(), 1)
+        self.runUntilSuccess(self.server.transport.joinGroup, "225.0.0.250")
+        self.server.transport.write("hello", ("225.0.0.250", self.server.transport.getHost().port))
+        reactor.iterate()
+        self.assertEquals(len(self.server.packets), 1)
+        self.server.transport.setLoopbackMode(0)
+        self.assertEquals(self.server.transport.getLoopbackMode(), 0)
+        self.server.transport.write("hello", ("225.0.0.250", self.server.transport.getHost().port))
+        reactor.iterate()
+        self.assertEquals(len(self.server.packets), 1)
     
     def testInterface(self):
         for o in self.client, self.server:
             self.assertEquals(o.transport.getOutgoingInterface(), "0.0.0.0")
             self.runUntilSuccess(o.transport.setOutgoingInterface, "127.0.0.1")
             self.assertEquals(o.transport.getOutgoingInterface(), "127.0.0.1")
-
+    
     def testJoinLeave(self):
         for o in self.client, self.server:
             self.runUntilSuccess(o.transport.joinGroup, "225.0.0.250")
@@ -289,7 +329,7 @@ class MulticastTestCase(unittest.TestCase):
         c = Server()
         p = reactor.listenMulticast(0, c)
         self.runUntilSuccess(self.server.transport.joinGroup, "225.0.0.250")
-        c.transport.write("hello world", ("225.0.0.250", self.server.transport.getHost()[2]))
+        c.transport.write("hello world", ("225.0.0.250", self.server.transport.getHost().port))
         
         iters = 0
         while iters < 100 and len(self.server.packets) == 0:
@@ -298,10 +338,37 @@ class MulticastTestCase(unittest.TestCase):
         self.assertEquals(self.server.packets[0][0], "hello world")
         p.stopListening()
 
-if not components.implements(reactor, interfaces.IReactorUDP):
+    def testMultiListen(self):
+        c = Server()
+        p = reactor.listenMulticast(0, c, listenMultiple=True)
+        self.runUntilSuccess(self.server.transport.joinGroup, "225.0.0.250")
+        portno = p.getHost().port
+        c2 = Server()
+        p2 = reactor.listenMulticast(portno, c2, listenMultiple=True)
+        self.runUntilSuccess(self.server.transport.joinGroup, "225.0.0.250")
+        c.transport.write("hello world", ("225.0.0.250", portno))
+
+        self.runReactor(0.4, True)
+        self.assertEquals(c.packets[0][0], "hello world")
+        self.assertEquals(c2.packets[0][0], "hello world")
+        p.stopListening()
+        p2.stopListening()
+
+    testMultiListen.skip = "on non-linux platforms it appears multiple processes can listen, but not multiple sockets in same process?"
+
+if not interfaces.IReactorUDP(reactor, None):
     UDPTestCase.skip = "This reactor does not support UDP"
 if not hasattr(reactor, "connectUDP"):
     OldConnectedUDPTestCase.skip = "This reactor does not support connectUDP"
-if not components.implements(reactor, interfaces.IReactorMulticast):
+if not interfaces.IReactorMulticast(reactor, None):
     MulticastTestCase.skip = "This reactor does not support multicast"
 
+def checkForLinux22():
+    import os
+    if os.path.exists("/proc/version"):
+        s = open("/proc/version").read()
+        if s.startswith("Linux version"):
+            s = s.split()[2]
+            if s.split(".")[:2] == ["2", "2"]:
+                MulticastTestCase.testInterface.im_func.todo = "figure out why this fails in linux 2.2"
+checkForLinux22()
