@@ -1,25 +1,14 @@
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# -*- test-case-name: twisted.test.test_util -*-
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 from twisted.trial import unittest
 
 from twisted.python import util
 from twisted.python.runtime import platformType
 import os.path, sys
-import shutil
+import shutil, errno
 
 class UtilTestCase(unittest.TestCase):
 
@@ -37,6 +26,35 @@ class UtilTestCase(unittest.TestCase):
             pass
         else:
             raise unittest.FailTest, "util.raises didn't raise when it should have"
+   
+    def testUninterruptably(self):
+        def f(a, b):
+            self.calls += 1
+            exc = self.exceptions.pop()
+            if exc is not None:
+                raise exc(errno.EINTR, "Interrupted system call!")
+            return a + b
+        
+        self.exceptions = [None]
+        self.calls = 0
+        self.assertEquals(util.untilConcludes(f, 1, 2), 3)
+        self.assertEquals(self.calls, 1)
+        
+        self.exceptions = [None, OSError, IOError]
+        self.calls = 0
+        self.assertEquals(util.untilConcludes(f, 2, 3), 5)
+        self.assertEquals(self.calls, 3)
+    
+    def testUnsignedID(self):
+        util.id = lambda x: x
+        try:
+            for i in range(1, 100):
+                self.assertEquals(util.unsignedID(i), i)
+            top = (sys.maxint + 1L) * 2L
+            for i in range(-100, -1):
+                self.assertEquals(util.unsignedID(i), top + i)
+        finally:
+            del util.id
 
 class OrderedDictTest(unittest.TestCase):
     def testOrderedDict(self):
@@ -261,3 +279,27 @@ class IntervalDifferentialTestCase(unittest.TestCase):
         d.removeInterval(5)
         self.assertEquals(d.next(), (10, None))
         self.assertRaises(ValueError, d.removeInterval, 10)
+
+
+class EQ(object, util.FancyEqMixin):
+    compareAttributes = ('a', 'b')
+    def __init__(self, a, b):
+        self.a, self.b = a, b
+
+class Bar(object):
+    pass
+
+class TestFancyEqMixin(unittest.TestCase):
+            
+    def testIsInstance(self):
+        eq = EQ(8, 9)
+        f = Bar()
+        self.failIfEqual(eq, f)
+
+    def testEquality(self):
+        ea, eb = EQ(1, 2), EQ(1, 2)
+
+        self.failUnlessEqual(ea, eb)
+        self.failUnlessEqual(eb, ea)
+        self.failUnlessEqual((ea != eb), False)
+        self.failUnlessEqual((eb != ea), False)
