@@ -1,19 +1,7 @@
 # -*- test-case-name: twisted.test.test_woven -*-
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2000-2002 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 __version__ = "$Revision: 1.53 $"[11:-2]
 
@@ -21,6 +9,8 @@ import types
 import weakref
 import warnings
 
+from zope.interface import implements
+
 from twisted.python import components, reflect
 from twisted.internet import defer
 
@@ -42,7 +32,7 @@ class Model:
     A Model which keeps track of views which are looking at it in order
     to notify them when the model changes.
     """
-    __implements__ = interfaces.IModel
+    implements(interfaces.IModel)
 
     def __init__(self, *args, **kwargs):
         if len(args):
@@ -243,6 +233,7 @@ class Model:
             if hasattr(self, 'parent') and self.parent:
                 self.parent.setSubmodel(request, self.name, data)
             self.orig = self.original = data
+components.backwardsCompatImplements(Model)
 
 
 class MethodModel(Model):
