@@ -1,20 +1,8 @@
 # -*- test-case-name: twisted.test.test_woven -*-
 #
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2000-2002 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 from __future__ import nested_scopes
 
@@ -33,6 +21,7 @@ from twisted.python import components
 from twisted.python import log
 from twisted.web import resource, microdom, html, error
 from twisted.web.server import NOT_DONE_YET
+from zope.interface import implements
 
 try:
     import cPickle as pickle
@@ -80,8 +69,7 @@ templateCache = {}
 
 
 class View:
-
-    __implements__ = (resource.IResource, interfaces.IView)
+    implements(resource.IResource, interfaces.IView)
     # wvfactory_xxx method signature: request, node, model; returns Widget
     # wvupdate_xxx method signature: request, widget, data; mutates widget 
     #    based on data (not necessarily an IModel; 
@@ -110,7 +98,8 @@ class View:
         A view must be told what its model is, and may be told what its
         controller is, but can also look up its controller if none specified.
         """
-        if not components.implements(m, interfaces.IModel):
+        components.fixClassImplements(m.__class__)
+        if not interfaces.IModel.providedBy(m):
             m = model.adaptToIModel(m, None, None)
         self.model = self.mainModel = m
         # It's the responsibility of the calling code to make sure
@@ -659,7 +648,7 @@ class View:
             request.write("</body></html>")
             request.finish()
         return failure
-
+components.backwardsCompatImplements(View)
 
 class LiveView(View):
     livePage = 1
@@ -690,7 +679,8 @@ def registerViewForModel(view, model):
     """
     components.registerAdapter(view, model, interfaces.IView)
 #     adapter = components.getAdapter(model, resource.IResource, None)
-#     if adapter is None and components.implements(view, resource.IResource):
+#     components.fixClassImplements(view.__class__)
+#     if adapter is None and resource.IResource.providedBy(view):
 #         components.registerAdapter(view, model, resource.IResource)
 
 
