@@ -1,20 +1,8 @@
-# -*- test-case-name: twisted.test.test_words -*-
+# -*- test-case-name: twisted.words.test.test_words -*-
+
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
 """
@@ -97,6 +85,7 @@ Then there's a whole bunch of things that would be nice to have.
 
 # System Imports
 import types, time
+from zope.interface import implements
 
 # Twisted Imports
 from twisted.spread import pb
@@ -212,7 +201,7 @@ class IWordsClient(components.Interface):
         """
 
 class WordsClient:
-    __implements__ = IWordsClient
+    implements(IWordsClient)
     """A stubbed version of L{IWordsClient}.
 
     Useful for partial implementations.
@@ -227,6 +216,7 @@ class WordsClient:
     def memberJoined(self, member, group): pass
     def memberLeft(self, member, group): pass
 
+components.backwardsCompatImplements(WordsClient)
 
 class Transcript:
     """I am a transcript of a conversation between multiple parties.
@@ -247,7 +237,7 @@ class IWordsPolicy(components.Interface):
         """ Get a Participant, given a name."""
 
 class NormalPolicy:
-    __implements__ = IWordsPolicy
+    implements(IWordsPolicy)
 
     def __init__(self, participant):
         self.participant = participant
@@ -257,6 +247,8 @@ class NormalPolicy:
     def lookUpParticipant(self, nick):
         return self.participant.service.getPerspectiveNamed(nick)
 
+components.backwardsCompatImplements(NormalPolicy)
+
 class Participant(pb.Perspective, styles.Versioned):
     def __init__(self, name):
         pb.Perspective.__init__(self, name)
