@@ -1,20 +1,8 @@
-# -*- test-case-name: twisted.test.test_tendril -*-
+# -*- test-case-name: twisted.words.test.test_tendril -*-
 # $Id: tendril.py,v 1.32 2003/01/08 10:34:29 acapnotic Exp $
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 """Tendril between Words and IRC servers.
 
@@ -52,7 +40,7 @@ from twisted import copyright
 from twisted.cred import authorizer, error
 from twisted.internet import defer, protocol
 from twisted.persisted import styles
-from twisted.protocols import irc
+from twisted.words.protocols import irc
 from twisted.python import log, reflect
 from twisted.words import service
 from twisted.spread.util import LocalAsyncForwarder
