@@ -1,20 +1,7 @@
 # -*- test-case-name: twisted.test.test_domish -*-
 #
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2005 Twisted Matrix Laboratories.
+# See LICENSE for details.
 
 from __future__ import generators
 
@@ -26,18 +13,25 @@ except ImportError:
     import StringIO
 
 def _splitPrefix(name):
-    """Internal method for splitting a prefixed Element name into its respective parts """
+    """ Internal method for splitting a prefixed Element name into its
+        respective parts """
     ntok = name.split(":", 1)
     if len(ntok) == 2:
         return ntok
     else:
         return (None, ntok[0])
 
+# Global map of prefixes that always get injected
+# into the serializers prefix map (note, that doesn't
+# mean they're always _USED_)
+G_PREFIXES = { "http://www.w3.org/XML/1998/namespace":"xml" }    
+
 class _Serializer:
     """ Internal class which serializes an Element tree into a buffer """
     def __init__(self, prefixes = None):
         self.cio = StringIO.StringIO()
         self.prefixes = prefixes or {}
+        self.prefixes.update(G_PREFIXES)
         self.prefixCounter = 0
 
     def getValue(self):
@@ -87,7 +81,7 @@ class _Serializer:
         for k,v in elem.attributes.items():
             # If the attribute name is a list, it's a qualified attribute
             if isinstance(k, types.TupleType):
-                write(" %s:%s='%s'" % (self.getPrefix[k[0]], k[1], escapeToXml(v, 1)).encode("utf-8"))
+                write((" %s:%s='%s'" % (self.getPrefix(k[0]), k[1], escapeToXml(v, 1))).encode("utf-8"))
             else:
                 write((" %s='%s'" % ( k, escapeToXml(v, 1))).encode("utf-8"))
 
@@ -115,6 +109,7 @@ class _ListSerializer:
     def __init__(self, prefixes = None):
         self.writelist = []
         self.prefixes = prefixes or {}
+        self.prefixes.update(G_PREFIXES)
         self.prefixCounter = 0
 
     def getValue(self):
@@ -191,13 +186,14 @@ class _ListSerializer:
 SerializerClass = _Serializer
 
 def escapeToXml(text, isattrib = 0):
-    """Escape text to proper XML form, per section 2.3 in the XML specification.
+    """ Escape text to proper XML form, per section 2.3 in the XML specification.
 
-     @type text: L{str}
-     @param text: Text to escape
+    @type text: L{str}
+    @param text: Text to escape
 
-     @type isattrib: L{Boolean}
-     @param isattrib: Triggers escaping of characters necessary for use as attribute values
+    @type isattrib: L{bool}
+    @param isattrib: Triggers escaping of characters necessary for use as
+                     attribute values
     """
     text = text.replace("&", "&amp;")
     text = text.replace("<", "&lt;")
@@ -223,14 +219,13 @@ def generateOnlyKlass(list, klass):
             yield n
 
 def generateElementsQNamed(list, name, uri):
-    """ Filters Element items in a list with matching name and URI
-    """
+    """ Filters Element items in a list with matching name and URI. """
     for n in list:
         if n.__class__ == Element and n.name == name and n.uri == uri:
             yield n
 
 def generateElementsNamed(list, name):
-    """ Filters Element items in a list with matching name, regardless of URI
+    """ Filters Element items in a list with matching name, regardless of URI.
     """
     for n in list:
         if n.__class__ == Element and n.name == name:
@@ -238,13 +233,12 @@ def generateElementsNamed(list, name):
 
 
 class SerializedXML(str):
-    """ Marker class for pre-serialized XML in the DOM """
+    """ Marker class for pre-serialized XML in the DOM. """
     pass
 
         
 class Namespace:
-    """ Convenience object for tracking namespace declarations
-    """
+    """ Convenience object for tracking namespace declarations. """
     def __init__(self, uri):
         self._uri = uri
     def __getattr__(self, n):
@@ -254,28 +248,28 @@ class Namespace:
 
 
 class Element(object):
-    """Object representing a container (a.k.a. tag or element) in an HTML or XML document.
+    """ Object representing a container (a.k.a. tag or element) in an HTML or XML document.
 
-    An Element contains a series of attributes (name/value pairs),
-    content (character data), and other child Element objects. When building a document
+    An Element contains a series of attributes (name/value pairs), content
+    (character data), and other child Element objects. When building a document
     with markup (such as HTML or XML), use this object as the starting point.
 
-    @type uri: C{str}
+    @type uri: L{str}
     @ivar uri: URI of this Element's name
 
-    @type defaultUri: C{str}
+    @type defaultUri: L{str}
     @ivar defaultUri: URI this Element exists within
 
-    @type name: C{str}
+    @type name: L{str}
     @ivar name: Name of this Element
 
-    @type children: C{list}
+    @type children: L{list}
     @ivar children: List of child Elements and content
 
-    @type parent: C{Element}
+    @type parent: L{Element}
     @ivar parent: Reference to the parent Element, if any.
 
-    @type attributes: C{dict}
+    @type attributes: L{dict}
     @ivar attributes: Dictionary of attributes associated with this Element.
 
     """
@@ -283,7 +277,8 @@ class Element(object):
     def __init__(self, qname, defaultUri = None, attribs = None):
         """
         @param qname: Tuple of (uri, name)
-        @param defaultUri: The default URI of the element; defaults to the URI specified in L{qname}
+        @param defaultUri: The default URI of the element; defaults to the URI
+                           specified in L{qname}
         @param attribs: Dictionary of attributes
         """
         self.uri, self.name = qname
@@ -319,40 +314,43 @@ class Element(object):
         return ""
 
     def _dqa(self, attr):
-        """Dequalify an attribute key as needed"""
+        """ Dequalify an attribute key as needed """
         if isinstance(attr, types.TupleType) and attr[0] == self.uri:
             return attr[1]
         else:
             return attr
 
     def getAttribute(self, attribname, default = None):
-        """Retrieve the value of attribname, if it exists """
+        """ Retrieve the value of attribname, if it exists """
         return self.attributes.get(attribname, default)
 
     def hasAttribute(self, attrib):
-        """Determine if the specified attribute exists """
+        """ Determine if the specified attribute exists """
         return self.attributes.has_key(self._dqa(attrib))
     
     def compareAttribute(self, attrib, value):
-        """Safely compare the value of an attribute against a provided value; None-safe. """
+        """ Safely compare the value of an attribute against a provided value.
+        
+        C{None}-safe.
+        """
         return self.attributes.get(self._dqa(attrib), None) == value
 
     def swapAttributeValues(self, left, right):
-        """Swap the values of two attribute"""
+        """ Swap the values of two attribute. """
         d = self.attributes
         l = d[left]
         d[left] = d[right]
         d[right] = l
 
     def addChild(self, node):
-        """Add a child to this Element"""
+        """ Add a child to this Element. """
         if node.__class__ == Element:
             node.parent = self
         self.children.append(node)
         return self.children[-1]
 
     def addContent(self, text):
-        """Add some text data to this element"""
+        """ Add some text data to this Element. """
         c = self.children
         if len(c) > 0 and isinstance(c[-1], types.StringTypes):
             c[-1] = c[-1] + text
@@ -361,7 +359,9 @@ class Element(object):
         return c[-1]
 
     def addElement(self, name, defaultUri = None, content = None):
-        """Add a new child Element to this Element; preferred method
+        """ Add a new child Element to this Element.
+        
+        Preferred method.
         """
         result = None
         if isinstance(name, type(())):
@@ -380,21 +380,22 @@ class Element(object):
         return result
 
     def addRawXml(self, rawxmlstring):
-        """Add a pre-serialized chunk o' XML as a child of this Element.
-        """
+        """ Add a pre-serialized chunk o' XML as a child of this Element. """
         self.children.append(SerializedXML(rawxmlstring))
 
     def addUniqueId(self):
-        """Add a unique (across a given Python session) id attribute to this Element"""
+        """ Add a unique (across a given Python session) id attribute to this
+            Element.
+        """
         self.attributes["id"] = "H_%d" % Element._idCounter
         Element._idCounter = Element._idCounter + 1
 
     def elements(self):
-        """Iterate across all children of this Element that are Elements"""
+        """ Iterate across all children of this Element that are Elements. """
         return generateOnlyKlass(self.children, Element)
 
     def toXml(self, prefixes = None, closeElement = 1):
-        """Serialize this Element and all children to a string """
+        """ Serialize this Element and all children to a string. """
         s = SerializerClass(prefixes)
         s.serialize(self, closeElement)
         return s.getValue()
@@ -419,158 +420,164 @@ def elementStream():
         es = ExpatElementStream()
         return es
     except ImportError:
+        if SuxElementStream is None:
+            raise Exception("No parsers available :(")
         es = SuxElementStream()
         return es
 
-from twisted.protocols import sux
-class SuxElementStream(sux.XMLParser):
-    def __init__(self):
-        self.connectionMade()
-        self.DocumentStartEvent = None
-        self.ElementEvent = None
-        self.DocumentEndEvent = None
-        self.currElem = None
-        self.rootElem = None
-        self.documentStarted = False
-        self.defaultNsStack = []
-        self.prefixStack = []
-        self.parse = self.dataReceived
-
-    def findUri(self, prefix):
-        # Walk prefix stack backwards, looking for the uri
-        # matching the specified prefix
-        stack = self.prefixStack
-        for i in range(-1, (len(self.prefixStack)+1) * -1, -1):
-            if prefix in stack[i]:
-                return stack[i][prefix]
-        return None
-
-    def gotTagStart(self, name, attributes):
-        defaultUri = None
-        localPrefixes = {}
-        attribs = {}
-        uri = None
-                    
-        # Pass 1 - Identify namespace decls
-        for k, v in attributes.items():
-            if k.startswith("xmlns"):
-                x, p = _splitPrefix(k)
-                if (x == None): # I.e.  default declaration
-                    defaultUri = v
-                else:
-                    localPrefixes[p] = v
-                del attributes[k]
-
-        # Push namespace decls onto prefix stack
-        self.prefixStack.append(localPrefixes)
-
-        # Determine default namespace for this element; if there
-        # is one
-        if defaultUri == None and len(self.defaultNsStack) > 0:
-            defaultUri = self.defaultNsStack[-1]
-                
-        # Fix up name
-        prefix, name = _splitPrefix(name)
-        if prefix == None: # This element is in the default namespace
-            uri = defaultUri
-        else:
-            # Find the URI for the prefix
-            uri = self.findUri(prefix)
-        
-        # Pass 2 - Fix up and escape attributes
-        for k, v in attributes.items():
-            p, n = _splitPrefix(k)
-            if p == None:
-                attribs[n] = v
-            else:
-                attribs[(self.findUri(p)), n] = unescapeFromXml(v)
-
-        # Construct the actual Element object
-        e = Element((uri, name), defaultUri, attribs)
-
-        # Save current default namespace
-        self.defaultNsStack.append(defaultUri)
-
-        # Document already started
-        if self.documentStarted:
-            # Starting a new packet
-            if self.currElem == None:
-                self.currElem = e
-            # Adding to existing element
-            else:
-                self.currElem = self.currElem.addChild(e)
-        # New document
-        else:
-            self.rootElem = e
-            self.documentStarted = True
-            self.DocumentStartEvent(e)
-
-    def gotText(self, data):
-        if self.currElem != None:
-            self.currElem.addContent(data)
-
-    def gotCData(self, data):
-        if self.currElem != None:
-            self.currElem.addContent(data)
-
-    def gotComment(self, data):
-        # Ignore comments for the moment
-        pass
-
-    entities = { "amp" : "&",
-                 "lt"  : "<",
-                 "gt"  : ">",
-                 "apos": "'",
-                 "quot": "\"" }
-
-    def gotEntityReference(self, entityRef):
-        # If this is an entity we know about, add it as content
-        # to the current element
-        if entityRef in SuxElementStream.entities:
-            self.currElem.addContent(SuxElementStream.entities[entityRef])
-
-    def gotTagEnd(self, name):
-        # Ensure the document hasn't already ended
-        if self.rootElem == None:
-            # XXX: Write more legible explanation
-            raise ParserError, "Element closed after end of document."
-        
-        # Fix up name
-        prefix, name = _splitPrefix(name)
-        if prefix == None:
-            uri = self.defaultNsStack[-1]
-        else:
-            uri = self.findUri(prefix)
-
-        # End of document
-        if self.currElem == None:
-            # Ensure element name and uri matches
-            if self.rootElem.name != name or self.rootElem.uri != uri:
-                raise ParserError, "Mismatched root elements"
-            self.DocumentEndEvent()
+try:
+    from twisted.web import sux
+except:
+    SuxElementStream = None
+else:
+    class SuxElementStream(sux.XMLParser):
+        def __init__(self):
+            self.connectionMade()
+            self.DocumentStartEvent = None
+            self.ElementEvent = None
+            self.DocumentEndEvent = None
+            self.currElem = None
             self.rootElem = None
+            self.documentStarted = False
+            self.defaultNsStack = []
+            self.prefixStack = []
+            self.parse = self.dataReceived
 
-        # Other elements
-        else:
-            # Ensure the tag being closed matches the name of the current
-            # element
-            if self.currElem.name != name or self.currElem.uri != uri:
-                # XXX: Write more legible explanation
-                raise ParserError, "Malformed element close"
+        def findUri(self, prefix):
+            # Walk prefix stack backwards, looking for the uri
+            # matching the specified prefix
+            stack = self.prefixStack
+            for i in range(-1, (len(self.prefixStack)+1) * -1, -1):
+                if prefix in stack[i]:
+                    return stack[i][prefix]
+            return None
 
-            # Pop prefix and default NS stack
-            self.prefixStack.pop()
-            self.defaultNsStack.pop()
+        def gotTagStart(self, name, attributes):
+            defaultUri = None
+            localPrefixes = {}
+            attribs = {}
+            uri = None
 
-            # Check for parent null parent of current elem;
-            # that's the top of the stack
-            if self.currElem.parent == None:
-                self.ElementEvent(self.currElem)
-                self.currElem = None
+            # Pass 1 - Identify namespace decls
+            for k, v in attributes.items():
+                if k.startswith("xmlns"):
+                    x, p = _splitPrefix(k)
+                    if (x == None): # I.e.  default declaration
+                        defaultUri = v
+                    else:
+                        localPrefixes[p] = v
+                    del attributes[k]
 
-            # Anything else is just some element wrapping up
+            # Push namespace decls onto prefix stack
+            self.prefixStack.append(localPrefixes)
+
+            # Determine default namespace for this element; if there
+            # is one
+            if defaultUri == None and len(self.defaultNsStack) > 0:
+                defaultUri = self.defaultNsStack[-1]
+
+            # Fix up name
+            prefix, name = _splitPrefix(name)
+            if prefix == None: # This element is in the default namespace
+                uri = defaultUri
             else:
-                self.currElem = self.currElem.parent
+                # Find the URI for the prefix
+                uri = self.findUri(prefix)
+
+            # Pass 2 - Fix up and escape attributes
+            for k, v in attributes.items():
+                p, n = _splitPrefix(k)
+                if p == None:
+                    attribs[n] = v
+                else:
+                    attribs[(self.findUri(p)), n] = unescapeFromXml(v)
+
+            # Construct the actual Element object
+            e = Element((uri, name), defaultUri, attribs)
+
+            # Save current default namespace
+            self.defaultNsStack.append(defaultUri)
+
+            # Document already started
+            if self.documentStarted:
+                # Starting a new packet
+                if self.currElem == None:
+                    self.currElem = e
+                # Adding to existing element
+                else:
+                    self.currElem = self.currElem.addChild(e)
+            # New document
+            else:
+                self.rootElem = e
+                self.documentStarted = True
+                self.DocumentStartEvent(e)
+
+        def gotText(self, data):
+            if self.currElem != None:
+                self.currElem.addContent(data)
+
+        def gotCData(self, data):
+            if self.currElem != None:
+                self.currElem.addContent(data)
+
+        def gotComment(self, data):
+            # Ignore comments for the moment
+            pass
+
+        entities = { "amp" : "&",
+                     "lt"  : "<",
+                     "gt"  : ">",
+                     "apos": "'",
+                     "quot": "\"" }
+
+        def gotEntityReference(self, entityRef):
+            # If this is an entity we know about, add it as content
+            # to the current element
+            if entityRef in SuxElementStream.entities:
+                self.currElem.addContent(SuxElementStream.entities[entityRef])
+
+        def gotTagEnd(self, name):
+            # Ensure the document hasn't already ended
+            if self.rootElem == None:
+                # XXX: Write more legible explanation
+                raise ParserError, "Element closed after end of document."
+
+            # Fix up name
+            prefix, name = _splitPrefix(name)
+            if prefix == None:
+                uri = self.defaultNsStack[-1]
+            else:
+                uri = self.findUri(prefix)
+
+            # End of document
+            if self.currElem == None:
+                # Ensure element name and uri matches
+                if self.rootElem.name != name or self.rootElem.uri != uri:
+                    raise ParserError, "Mismatched root elements"
+                self.DocumentEndEvent()
+                self.rootElem = None
+
+            # Other elements
+            else:
+                # Ensure the tag being closed matches the name of the current
+                # element
+                if self.currElem.name != name or self.currElem.uri != uri:
+                    # XXX: Write more legible explanation
+                    raise ParserError, "Malformed element close"
+
+                # Pop prefix and default NS stack
+                self.prefixStack.pop()
+                self.defaultNsStack.pop()
+
+                # Check for parent null parent of current elem;
+                # that's the top of the stack
+                if self.currElem.parent == None:
+                    self.ElementEvent(self.currElem)
+                    self.currElem = None
+
+                # Anything else is just some element wrapping up
+                else:
+                    self.currElem = self.currElem.parent
 
 
 class ExpatElementStream:
@@ -586,7 +593,7 @@ class ExpatElementStream:
         self.parser.StartNamespaceDeclHandler = self._onStartNamespace
         self.parser.EndNamespaceDeclHandler = self._onEndNamespace
         self.currElem = None
-        self.defaultNsStack = []
+        self.defaultNsStack = [None]
         self.documentStarted = 0        
 
     def parse(self, buffer):
@@ -595,11 +602,14 @@ class ExpatElementStream:
     def _onStartElement(self, name, attrs):
         # Generate a qname tuple from the provided name
         qname = name.split(" ")
+        if len(qname) == 1:
+            qname = (None, name)
 
         # Process attributes
         for k, v in attrs.items():
             if k.find(" ") != -1:
-                attrs[k.split(" ")] = v
+                aqname = k.split(" ")
+                attrs[(aqname[0], aqname[1])] = v
                 del attrs[k]
 
         # Construct the new element
